/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.losersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "audio.h"


/*!
 * \brief Inicializa el subsistema de Audio
 * \return mdulo de Audio
 */
Audio * audio_init (int enabled)
{
	Audio * data;

	data = (Audio *) malloc (sizeof (Audio));

	if (! data)
	{
		util_print_fail_module ("Audio");
		exit (1);
	}

	if (! enabled)
	{
		data->enabled = FALSE;
		data->sound_volume = 0;
		return data;
	}
	

	if (SDL_InitSubSystem (SDL_INIT_AUDIO) == -1)
	{
		printf (_("Error: %s\n"), Mix_GetError());
		printf (_("Audio will be disabled\n"));
		data->enabled = FALSE;
	}
	else
	{
		/* Si no puede acceder al recurso de Audio del sistema 
		 * intenta continuar ejecutando el programa pero evitando
		 * cargar o reproducir sonidos.
		 */
		if (Mix_OpenAudio (44100, AUDIO_S16SYS, 2, 2048))
		{
			printf (_("Error: %s\n"), Mix_GetError());
			printf (_("Audio will be disabled\n"));
			
			data->enabled = FALSE;
		}
		else
		{
			data->enabled = TRUE;
			Mix_AllocateChannels (32);
			audio_load_sounds (data);
			audio_set_sound_volume (data, 128);
		}
	}

	return data;
}

/*!
 * \brief Carga todos los sonidos
 * \param data mdulo principal
 */
void audio_load_sounds (Audio * data)
{
	int i;

	for (i = 0; i < _SND_MAX; i ++)
		data->sounds [i] = NULL;

	for (i = 0; i < _SND_MAX; i ++)
		audio_load_sound (data, i);
}

/*!
 * \brief Carga un sonido en base a su identificador
 * \param data mdulo principal
 * \param id identificador del sonido, por ejemplo `2` es equivalente a cargar el archivo `/sounds/2.wav`. Evite utilizar enteros, para ello est la enumeracin sound_id
 */
void audio_load_sound (Audio * data, enum sound_id id)
{
	char path_to_file [1024];

	sprintf (path_to_file, DATADIR "/sounds/%i.wav", id);
	
	if (data->sounds [id] == NULL)
	{
		data->sounds [id] = Mix_LoadWAV (path_to_file);

		if (! data->sounds [id])
			printf (_("Error: %s\n"), Mix_GetError ());
	}
	else
	{
		printf (_("Warning, the sound identified by code %d was already load\n"), id);
	}
}

void audio_play_sound (Audio * data, enum sound_id id)
{
	if (! data->enabled)
		return;
	
	if (data->sounds [id])
	{
		if (Mix_PlayChannel (-1, data->sounds [id], 0) == -1)
		{
			Mix_PlayChannel (0, data->sounds [id], 0);
		
		/*	printf (_("Error, can't play sound identified by code %d. SDL_mixer report: %s\n"), id, Mix_GetError ());*/
		}
	}
	else
	{
		printf (_("Warning: the sound identified by code %d is no avariable\n"), id);
	}
}

void audio_set_sound_volume (void * _data, int volume)
{
	Audio * data = (Audio *) _data;

	if (data->enabled)
	{
		Mix_Volume (-1, volume);
		data->sound_volume = volume;
	}
}

void audio_free (Audio * data)
{
	int i;

	if (data->enabled)
	{
		for (i = 0; i < _SND_MAX; i ++)
		{
			if (data->sounds [i])
			{
				Mix_FreeChunk (data->sounds [i]);
				data->sounds [i] = NULL;
			}
		}
		
		Mix_CloseAudio ();
	}
	
	free (data);
}

int audio_get_sound_volume (Audio * data)
{
	return data->sound_volume;
}


int audio_is_enabled (Audio * data)
{
	return (data->enabled == TRUE);
}
