/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.losersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "ball.h"

/*!
 * \brief inicia el mdulo gestor de la pelota
 * \param position indica cual es el jugador que tiene el control inicial de
 * la pelota, los valores vlidos aqu sern 0 o 1
 * \param debug indica si se deben imprimir las lineas gua
 */
Ball * ball_init (Game * game, int debug)
{
	Ball * data;

	data = (Ball *) malloc (sizeof (Ball));

	if (! data)
	{
		util_print_fail_module ("Ball");
		exit (1);
	}
	
	data->game = game;
	data->step = 0;
	data->collision = 0;
	
	data->image = util_load_image (DATADIR "/ball.png", FALSE);

	if (! data->image)
	{
		util_print_fail_module ("Ball");
		exit (1);
	}

	data->collision_image = util_load_image (DATADIR "/ball_collision.png",
			FALSE);

	if (! data->collision_image)
	{
		util_print_fail_module ("Ball");
		exit (1);
	}

	data->position.ratio = data->image->h / 2;
	data->position.vx = 0.0;
	data->position.vy = 0.0;
	data->position.power = 3.0;
	
	data->player_start = util_rand (0, 1);
	ball_init_restart (data);
	
	ball_change_state (data, NULL, ball_state_restart);

	if (debug)
	{
		ball_draw_debug_lines (data);
		data->debug = debug;
	}
	else
		data->debug = FALSE;

	ball_clear_path (data);
	
	return data;
}


/*!
 * \brief Actualizacin lgica
 * \param data mdulo principal
 */
void ball_update (Ball * data)
{
	data->state (data);
	ball_animation_update (data);
	
	if (data->collision)
		data->collision --;
}


/*!
 * \brief Actualiza la animacin de la pelota
 * \param data mdulo principal
 */
void ball_animation_update (Ball * data)
{
	data->step += data->position.vx / 4;

	if (data->step < 0.0)
		data->step = 23.0;
	else
	{
		if (data->step > 23.0)
			data->step = 0;
	}
}


/*!
 * \brief imprime la pelota en pantalla
 * \param data mdulo principal
 * \param display gestor de pantalla
 */
void ball_draw (Ball * data, Display * display)
{
	int x = data->position.x - data->position.ratio;
	int y = data->position.y - data->position.ratio;
	SDL_Surface * image;

	if (data->collision)
		image = data->collision_image;
	else
		image = data->image;


	if (data->debug)
		ball_draw_debug_path (data, display);

	display_draw (display, image, data->step, 24, x, y, FALSE);
}


/*!
 * \brief termina con el mdulo
 * \param data mdulo principal
 */
void ball_free (Ball * data)
{
	SDL_FreeSurface (data->image);
	SDL_FreeSurface (data->collision_image);
	free (data);
}

/*!
 * \brief inicia el estado `bouncing` (rebotando)
 * \param data mdulo principal
 */
void ball_init_bouncing (Ball * data)
{
	ball_generate_path (data);
}


/*!
 * \brief inicia el estado `restart` (reinicia)
 * \param data mdulo principal
 */
void ball_init_restart (Ball * data)
{
	data->delay = 128;
	data->position.vx = 0;

	data->final_y = 280;
	data->starting = TRUE;
	
	if (data->player_start)
		data->final_x = 320 + 160;
	else
		data->final_x = 160;

	data->angle = 0.0;

	ball_clear_path (data);
}


/*!
 * \brief inicia el estado `ready` (esperando a iniciar)
 * \param data mdulo principal
 */
void ball_init_ready (Ball * data)
{
	data->position.power = 3;
	data->starting = FALSE;
}

/*!
 * \brief inicia el estado `goal` (cuando la pelota toca el suelo)
 * \param data mdulo principal
 */
void ball_init_goal (Ball * data)
{
	int player_win;
	
	if (data->position.x < 320)
		player_win = 1;
	else
		player_win = 0;

	/* solo aumenta un tanto al jugador si fu el que realiz el saque */
	if (data->player_start == player_win)
		status_bar_goal (data->game->status_bar, player_win);

	data->player_start = player_win;
	
	data->delay = 128;
}


/*!
 * \brief gestiona el estado `goal` (mientras la pelota desaparece)
 * \param data mdulo principal
 */
void ball_state_goal (Ball * data)
{
	int flags = SDL_RLEACCEL | SDL_SRCALPHA;

	data->starting = TRUE;
	
	data->position.x += data->position.vx;
	data->position.y += data->position.vy;
	
	data->position.vy += 0.04;
	
	ball_check_boundary_collision (& data->position);
	
	if (data->delay > 0)
	{
		data->delay --;
		SDL_SetAlpha (data->image, flags, data->delay * 2);
	}
	else
	{
		ball_change_state (data, ball_init_restart, ball_state_restart);
	}
}

/*!
 * \brief gestor del estado `bouncing` (rebotando)
 * \param data mdulo principal
 */
void ball_state_bouncing (Ball * data)
{
	data->position.x += data->position.vx;
	data->position.y += data->position.vy;

	data->position.vy += 0.04;
	
	if (data->position.vy > 0.0 && data->collision < 1)
		ball_collision (data);

	/* si la pelota toca el suelo ... */
	if (ball_check_boundary_collision (& data->position))
		ball_change_state (data, ball_init_goal, ball_state_goal);
}


/*!
 * \brief gestor del estado `restart` (reinicia)
 * \param data mdulo principal
 */
void ball_state_restart (Ball * data)
{
	int flags = SDL_RLEACCEL | SDL_SRCALPHA;

	if (data->delay < 1)
	{
		ball_change_state (data, ball_init_ready, ball_state_ready);
	}
	else
	{	
		data->delay --;
		data->angle += 0.1;

		data->position.x = cos(data->angle)* data->delay+ data->final_x;
		data->position.y = sin(data->angle)* data->delay+ data->final_y;

		SDL_SetAlpha (data->image, flags, 255 - data->delay * 2);
	}
}


/*!
 * \brief gestor del estado `ready` (esperando iniciar)
 * \param data mdulo principal
 */
void ball_state_ready (Ball * data)
{
	if (ball_collision (data))
	{
		ball_change_state (data, ball_init_bouncing, 
				ball_state_bouncing);
	}
}


/*!
 * \brief cambia el estado actual de la pelota
 * \param mdulo principal
 * \param init funcin de inicio de estado
 * \param update funcin que gestiona la actualizacin del estado
 */
void ball_change_state (Ball * data, void (* init) (Ball * data), 
		void (* update) (Ball * data))
{
	if (init)
		init (data);

	data->state = update;
}

/*!
 * \brief Imprime lineas guias para el modo depuracin
 * \param data mdulo principal
 */
void ball_draw_debug_lines (Ball * data)
{
	Uint32 color = SDL_MapRGB (data->image->format, 255, 120, 120);
	int i;
	int dx = 0;
	int ratio = data->position.ratio;

	for (i = 0; i < 24; i ++)
	{
		util_draw_circle (data->image, color, dx + ratio, ratio, ratio);
		dx += ratio * 2;
	}
}


/*!
 * \brief Verifica si la pelota se encuentra en colisin con los personajes
 * \param data mdulo principal
 * \return TRUE si colision con un personaje
 */
int ball_collision (Ball * data)
{
	int ratio = data->position.ratio;
	int x = data->position.x;
	int y = data->position.y;
	int px;
	int py;
	int pratio;
	Player * player;

	if (data->position.x < 320)
		player = data->game->player_1;
	else
		player = data->game->player_2;

	player_get_collision_area (player, & px, & py, & pratio);

	if (util_is_collision (x, y, px, py, pratio + ratio))
	{
		float angle;
		int dx = x - px;

		angle = util_get_angle (x, y, px, py);
	
		data->position.power *= player_get_collision_power (player, dx);

		ball_position_fix_power (& data->position);
		
		
		data->position.vx = cos (angle) * data->position.power;
		data->position.vy = - sin (angle) * data->position.power;

		ball_position_separate (& data->position, px, py, pratio);

		data->collision = 25;

		/* informa al jugador que ha colisionado con la pelota */
		player_collision_with_ball (player, dx);
		
		/* reproduce un sonido de la colision */
		audio_play_sound (data->game->world->audio, SND_COLLISION_2);
	
		/* genera el nuevo camino */
		ball_generate_path (data);
		
		return TRUE;
	}

	return FALSE;
}


int ball_check_boundary_collision (BallPosition * position)
{
	/* verifica los lmites derecho e izquierdo */
	if (position->vx > 0)
	{
		if (position->x + position->vx + position->ratio > 640)
		{
			position->x = 640 - position->ratio;
			position->vx *= -1;
			return FALSE;
		}
	}
	else
	{
		if (position->x + position->vx - position->ratio < 0)
		{
			position->x = position->ratio;
			position->vx *= -1;
			return FALSE;
		}
	}


	/* verifica los lmites superior e inferior */
	if (position->vy < 0)
	{
		if (position->y + position->vy - position->ratio < 0)
		{
			position->y = position->ratio;
			position->vy *= -1;

//			audio_play_sound (data->audio, SND_TOUCH);
			return FALSE;
		}
	}
	else
	{
		if (position->y + position->vy + position->ratio > FLOOR)
		{
			position->y = FLOOR - position->ratio;
			position->vy /= -2;
			position->vx /= 2;

//			audio_play_sound (data->audio, SND_TOUCH);
			return TRUE;
		}
	}

	/* Veritica si colisiona con la barra central */
	if (position->y < PILLAR_Y)
	{
		if (util_is_collision (position->x, position->y, 
					PILLAR_X, PILLAR_Y, 
					position->ratio + PILLAR_RATIO))
		{
			float angle;
			
			angle = util_get_angle (position->x, position->y, 
					PILLAR_X, PILLAR_Y);
			
			ball_position_separate (position, PILLAR_X, PILLAR_Y, 
					PILLAR_RATIO);

			position->vx = cos (angle) * position->power * 0.8;
			position->vy = - sin (angle) * position->power * 0.8;

			if (position->vx < 0.25 && position->vx > -0.25)
			{
				if (position->vx < 0.0)
					position->vx = -0.25;
				else
					position->vx = 0.25;
			}

//			audio_play_sound (data->audio, SND_TOUCH);
		}
	}
	else
	{
		// colisiona con el poste del lado izquierdo
		if (util_include_value (position->x + position->ratio, 
					PILLAR_X - PILLAR_RATIO,
					PILLAR_X + PILLAR_RATIO))
		{
			position->x = PILLAR_X - PILLAR_RATIO - position->ratio - 1;
			position->vx *= -1;
//			audio_play_sound (position->audio, SND_TOUCH);
			return FALSE;
		}
		
		// colisiona con el poste del lado derecho
		if (util_include_value (position->x - position->ratio, 
					PILLAR_X - PILLAR_RATIO,
					PILLAR_X + PILLAR_RATIO))
		{
			position->x = PILLAR_X + PILLAR_RATIO + position->ratio + 1;
			position->vx *= -1;
//			audio_play_sound (position->audio, SND_TOUCH);
			return FALSE;
		}
						
	}
	
	return FALSE;
}


/*!
 * \brief Evita que la pelota y una circunferencia se solapen
 * \param data mdulo principal
 * \param px posicin coordenada horizontal de la circunferencia
 * \param py posicin coordenada vertical de la circunferencia
 * \param pratio radio de la circunferencia
 */
void ball_position_separate (BallPosition * position, int px, int py, int ratio)
{
	float angle = util_get_angle (position->x, position->y, px, py);
	int distance;
	int total_ratio = ratio + position->ratio;

	/* separa la pelota del personaje */
	distance = total_ratio - util_get_distance (position->x, position->y, 
			px, py);

	position->y -= distance * sin (angle);
	position->x += distance * cos (angle);
}


/*!
 * \brief Imprime la trayectoria de la pelota
 * \param data mdulo principal
 * \param display mdulo gestor de impresiones en pantalla
 */
void ball_draw_debug_path (Ball * data, Display * display)
{
	int i;
	int x;
	int y;

	for (i = 0; i < MAX_PATH_POINTS; i ++)
	{
		x = data->path [i].x;
		y = data->path [i].y;
		
		if (x == 0 && y  == 0)
			return;
		else
			display_put_big_pixel (display, x, y);
	}
}


/*!
 * \brief Reinicia las coordenadas de la trayectoria de la pelota
 * \param data mdulo principal
 */
void ball_clear_path (Ball * data)
{
	data->path [0].x = 0;
	data->path [0].y = 0;
	data->elements_in_path_array = 0;
}



/*!
 * \brief Genera el recorrido de la pelota
 * \param data mdulo principal
 */
void ball_generate_path (Ball * data)
{
	BallPosition position = data->position;
	int scale = 5;
	int i;
	int j;

	
	for (i = 0; i < MAX_PATH_POINTS; i ++)
	{
		for (j = 0; j < scale; j ++)
		{
			position.x += position.vx;
			position.y += position.vy;

			position.vy += 0.04;

			/* si la pelota toca el suelo ... */
			if (ball_check_boundary_collision (& position))
			{
				data->path [i].x = position.x;
				data->path [i].y = position.y;

				if (i < MAX_PATH_POINTS - 1)
				{
					data->path [i].x = 0;
					data->path [i].y = 0;
					data->elements_in_path_array = i;
				}

				return;
			}
		}
		
		data->path [i] = position;
	}
}

/*!
 * \brief Limita el poder de la pelota
 * \param position estado interno de la pelota
 */
void ball_position_fix_power (BallPosition * position)
{
	if (position->power > 9.0)
		position->power = 9.0;
	else
	{
		if (position->power < 2.0)
			position->power = 2.0;
	}
}


/*!
 * \brief Informa la coordenada horizontal de la pelota
 * \param data mdulo principal
 * \return coordenada x
 */
int ball_get_x (Ball * data)
{
	return (int) data->position.x;
}

/*!
 * \brief Informa la coordenada vertical de la pelota
 * \param data mdulo principal
 * \return coordenada y
 */
int ball_get_y (Ball * data)
{
	return (int) data->position.y;
}

/*!
 * \brief indica si la pelota est iniciando (no se puede colisionar con ella)
 * \param data mdulo principal
 * \return TRUE si est iniciando
 */
int ball_are_starting (Ball * data)
{
	return data->starting;
}


/*!
 * \brief Indica si la pelota est lista para una colisin
 * \param data mdulo principal
 * \return TRUE si puede colisionar
 */
int ball_are_ready (Ball * data)
{
	return (! ball_are_starting (data));
}

/*!
 * \brief Obtiene la coordenada x del punto en donde la pelota toca el suelo
 * \param data mdulo principal
 * \return coordenada
 */
int ball_get_last_x_position_in_path (Ball * data)
{
	return data->path [data->elements_in_path_array - 1].x;
}

/*!
 * \brief Obtiene la coordenada y del punto en donde la pelota toca el suelo
 * \param data mdulo principal
 * \return coordenada
 */
int ball_get_last_y_position_in_path (Ball * data)
{
	return data->path [data->elements_in_path_array - 1].y;
}


/*!
 * \brief Obtiene la posicin de la trayectoria mas cercana al punto (x, y). Siempre y cuando la pelota est bajando.
 * \param data mdulo principal
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \return posicin mas cercana
 */
BallPosition ball_get_near_position_in_path (Ball * data, int x, int y)
{
	int best_index = -1;
	int best_distance = 5000;
	int actual_distance;
	int i;
	
	for (i = 0; i < data->elements_in_path_array; i ++)
	{
		if (data->path [i].vy > 0.0)
		{
			actual_distance = util_get_distance (x, y, 
					data->path [i].x, data->path [i].y);

			if (actual_distance < best_distance)
			{
				best_distance = actual_distance;
				best_index = i;
			}
		}
	}

	return data->path [best_index];
}


/*!
 * \brief Retorna una posicin de la trayectoria adecuada para golpear la pelota
 * \param data mdulo principal
 * \param min_height
 * \return 
 */
BallPosition ball_get_punchable_position (Ball * data, int min_height)
{
	int i;
	
	for (i = 0; i < data->elements_in_path_array; i ++)
	{
		if (data->path [i].vy > 0.0)
		{
			if (data->path [i].y > min_height)
				return data->path [i];
		}
	}

	/* ERROR */
	printf ("Error in ball_get_punchable_position\n");
	return data->path [0];
}

/*!
 * \brief Notifica si la trayectoria de la pelota no est disponible
 * \param data mdulo principal
 * \return TRUE si la trayectoria no est disponible
 */
int ball_no_path_available (Ball * data)
{
	return data->elements_in_path_array == 0;
}
