/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.losersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef _BALL_H
#define _BALL_H

#include "common.h"
#include "display.h"
#include "status_bar.h"
#include "audio.h"

/*! \brief cantidad de posiciones para registrar el recorrido de la pelota */
#define MAX_PATH_POINTS 128


/*!
 * \brief Utilizado para representar un 'instante' en la trayectoria de la
 * pelota. Se utiliza tanto en el vector `path` como en el atributo 
 * `position` que indica la posicin actual de la pelota
 */
typedef struct position_t
{
	float x;
	float y;
	float vx;
	float vy;
	float power;
	int ratio;
} BallPosition;



/*!
 * \brief Representa la pelota del partido de volleyball (escena Game)
 * 
 * 
 * La pelota tiene una animacin de giro muy sencilla, el atributo `image` 
 * contiene una grilla con varios grficos de la pelota con diferentes grados
 * de inclinacin. En cada actualizacin lgica (ver `ball_update`) se invoca 
 * a la funcin `ball_animation_update` que altera la velocidad de 
 * animacin conforme a la componente horizontal de velocidad `vx`, esto
 * produce la sensacin de giro y velocidad.
 */
typedef struct ball
{
	struct game_t * game;
	BallPosition position;
	
	int starting;
	float step;		/*!< cuadro de animacin actual */
	int delay;		/*!< contador general */ 
	float angle;		/*!< ngulo de giro para el estado `restart` */
	int final_x;		/*!< destino de la pelota cuando inicia */
	int final_y;		/*!< destino de la pelota cuando inicia */
	int collision;		/*!< contador que indica si la pelota 
				  colision recientemente (collision > 0), ver 
				  estado `bouncing` */
	int player_start;	/*!< jugador que realiz el ltimo `saque` */

	SDL_Surface * image;		/*!< cuadros de animacin */
	SDL_Surface * collision_image;	/*!< cuadros de animacin en colision */
	
	void (* state) (struct ball * data);	/*!< funcin gestor de estado */
	
	BallPosition path [MAX_PATH_POINTS];	/*!< trayectoria a realizar */ 
	int elements_in_path_array;		/*!< cantidad de elementos */
	
	int debug;				/*!< si el programa est en 
						  modo depuracin */
} Ball;

#include "player.h"
#include "game.h"

Ball * ball_init (struct game_t * game, int debug);

void ball_update (Ball * data);
void ball_draw (Ball * data, Display * display);
void ball_animation_update (Ball * data);
void ball_free (Ball * data);

void ball_init_bouncing (Ball * data);
void ball_init_restart (Ball * data);
void ball_init_ready (Ball * data);
void ball_init_goal (Ball * data);

void ball_state_bouncing (Ball * data);
void ball_state_restart (Ball * data);
void ball_state_ready (Ball * data);
void ball_state_goal (Ball * data);

void ball_change_state (Ball * data, void (* init) (Ball * data), 
		void (* update) (Ball * data));
void ball_draw_debug_lines (Ball * data);
int ball_collision (Ball * data);
int ball_check_boundary_collision (BallPosition * position);

void ball_position_separate (BallPosition * position, int px, int py, int r);

void ball_draw_debug_path (Ball * data, Display * display);
void ball_clear_path (Ball * data);
void ball_generate_path (Ball * data);
void ball_position_fix_power (BallPosition * position);
int ball_get_x (Ball * data);
int ball_get_y (Ball * data);


/* Utils for IA behavior */
int ball_are_starting (Ball * data);
int ball_are_ready (Ball * data);
int ball_get_last_x_position_in_path (Ball * data);
int ball_get_last_y_position_in_path (Ball * data);
BallPosition ball_get_near_position_in_path (Ball * data, int x, int y);
BallPosition ball_get_punchable_position (Ball * data, int min_height);
int ball_no_path_available (Ball * data);

#endif
