/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "characters.h"
#include "player.h"

/*!
 * \brief Inicia el mdulo buscando personajes
 * \return el mdulo creado
 */
Characters * characters_init (Options * options)
{
	Characters * data;

	data = (Characters *) malloc (sizeof (Characters));

	if (!data)
	{
		util_print_fail_module ("Characters");
		exit (1);
	}

	data->list = NULL;

	data->options = options;
	characters_start_scan (data);

	if (characters_count_players (data) < 2)
	{
		printf (_("Error: You need at least 2 characters in "
					"order to play this game\n"));
		characters_free (data);
		util_print_fail_module ("Characters");
		exit (1);
	}


	
	return data;
}


/*!
 * \brief Comienza a examinar directorios en busca de nuevos personajes
 * \param data mdulo principal
 */
void characters_start_scan (Characters * data)
{
	printf ("\n");

	characters_scan_dir (data, DATADIR "/characters");

#ifndef WIN32
	char * home = getenv ("HOME");

	if (home)
	{
		char buffer [1024];

		sprintf (buffer, "%s/%s", home, ".volleyball/characters");
		characters_scan_dir (data, buffer);
	}
#endif
}


/*!
 * \brief Examina un directorio en busca de personajes
 * \param data mdulo principal
 * \param path ruta del directorio `characters`
 */
void characters_scan_dir (Characters * data, char * path)
{
	DIR * dir;
	struct dirent * element;

	printf (_("Searching characters in `%s`\n"), path);
	
	dir = opendir (path);

	if (! dir)
	{
		printf (_("\tThe file %s does not exist\n"), path);
		printf ("\n");
		return;
	}

	for (element = readdir (dir); element; element = readdir (dir))
	{
		char * file = element->d_name;

		if (strcmp (file, ".") && strcmp (file, ".."))
		{
			characters_new_player (data, path, file);
		}
	}

	printf ("\n");
	closedir (dir);
}


/*!
 * \brief Intenta cargar un nuevo personaje desde el directorio indicado
 * \param data mdulo principal
 * \param path ruta del directorio `characters`
 * \param file nombre del directorio que contiene el personaje
 */
void characters_new_player (Characters * data, char * path, char * file)
{
	Player * new_player;
	
	new_player = player_init (path, file, data->options->debug);

	if (new_player)
		data->list = node_add (data->list, new_player);
}


/*!
 * \brief Elimina el mdulo de personajes
 * \param data mdulo principal
 */
void characters_free (Characters * data)
{
	node_free_all (data->list, player_free);
}


/*!
 * \brief Cuenta la cantidad de jugadores disponibles para utilizar
 * \param data mdulo principal
 * \return cantidad de personajes
 */
int characters_count_players (Characters * data)
{
	int n = 0;

	node_iterator_next (data->list);

	while (node_iterator_next (NULL))
		n ++;

	return n;
}


/*!
 * \brief Obtiene una referencia a un personaje del grupo
 * \param data mdulo principal
 * \param index ndice del personaje contando desde 0
 * \return personaje indicado o NULL en caso de error
 */
Player * characters_get_player (Characters * data, int index)
{
	Player * player;
	int n = 0;

	node_iterator_next (data->list);

	player = (Player *) node_iterator_next (NULL);

	while (player)
	{
		if (n == index)
			return player;

		n ++;
		player = (Player *) node_iterator_next (NULL);
	}

	return NULL;
}


Player * characters_get_random_player (Characters * data)
{
	int index = util_rand (0, characters_count_players (data) - 1);

	return characters_get_player (data, index);
}
