/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "cursor.h"
#include "select_vs.h"
#include "select.h"

/*!
 * \brief inicia el mdulo
 * \param player indice del jugador
 * \param cantidad mxima de personajes en la grilla
 */
Cursor * cursor_init (World * world, int player, int max_characters)
{
	Cursor * data;

	data = (Cursor *) malloc (sizeof (Cursor));

	if (! data)
	{
		util_print_fail_module ("Cursor");
		exit (1);
	}

	data->world = world;
	data->max_characters = max_characters;
	data->player_number = player;

	if (player)
	{
		data->step = PI;
		data->control = world->player_control_2;
	}
	else
	{
		data->step = 0.0;
		data->control = world->player_control_1;
	}

	data->sin_image = 0.0;
	data->active = TRUE;
	
	data->image = util_load_image (DATADIR "/cursor.png", TRUE);

	if (! data->image)
	{
		util_print_fail_module ("Cursor");
		exit (1);
	}

	cursor_set_start_position (data);
	
	return data;
}


/*!
 * \brief define la posicin inicial de cada cursor
 * \param data mdulo principal
 * \param player indice del jugador (0 o 1)
 */
void cursor_set_start_position (Cursor * data)
{
	data->y = 0;
	
	if (data->player_number)
	{
		data->x = 640;
		
		if (data->max_characters > 4)
			cursor_move_to (data, 4);
		else
			cursor_move_to (data, data->max_characters - 1);
	}
	else
	{
		data->x = 0;
		cursor_move_to (data, 0);
	}
}


/*!
 * \brief altera la posicin del cursor
 * \param data mdulo principal
 * \param character_index indice del personaje
 */
void cursor_move_to (Cursor * data, int character_index)
{
	if (character_index >= 0 && character_index < data->max_characters)
	{
		data->index = character_index;
		cursor_change_player (data);
	}

	data->end_x = 160 + (data->index % COLUMNS) * 64 - data->image->h / 2;
	data->end_y = 200 + (data->index / COLUMNS) * 64 - data->image->h / 2;
}


/*!
 * \brief actualiza la posicin del curso y procesa los eventos del usuario
 * \param data mdulo principal
 */
void cursor_update (Cursor * data)
{
	data->y += (data->end_y - data->y) / 2;
	data->x += (data->end_x - data->x) / 2;

	if (data->active)
	{
		if (player_control_query_no_repeat (data->control, UP))
			cursor_move_to (data, data->index - COLUMNS);
		
		if (player_control_query_no_repeat (data->control, DOWN))
			cursor_move_to (data, data->index + COLUMNS);
		
		if (player_control_query_no_repeat (data->control, LEFT))
			cursor_move_to (data, data->index - 1);
		
		if (player_control_query_no_repeat (data->control, RIGHT))
			cursor_move_to (data, data->index + 1);

		if (player_control_query_no_repeat (data->control, SELECT))
			cursor_select (data);

		data->step += 0.2;
		data->sin_image = 5 * sin (data->step) + 22;
	}
}


/*!
 * \brief imprime el cursor en pantalla
 * \param data mdulo principal
 * \param display mdulo de pantalla
 */
void cursor_draw (Cursor * data, Display * display)
{
	int x0 = data->x - data->sin_image;
	int y0 = data->y - data->sin_image;
	int x1 = data->x + data->sin_image;
	int y1 = data->y + data->sin_image;
	int offset_image;
	int offset_number; 

	if (data->active)
	{
		offset_image = data->player_number * 4;
		offset_number = data->player_number + 8;
	}
	else
	{
		offset_image = 10;
		offset_number = data->player_number + 14;
	}

	/* dibuja las esquinas del cursor */
	display_draw (display, data->image, offset_image + 0, 16, x0, y0, 0);
	display_draw (display, data->image, offset_image + 1, 16, x1, y0, 0);
	display_draw (display, data->image, offset_image + 2, 16, x0, y1, 0);
	display_draw (display, data->image, offset_image + 3, 16, x1, y1, 0);
	
	/* identificador del jugador */
	display_draw (display, data->image, offset_number, 16, data->x, 
			data->y, 0);

	cursor_draw_player_nick (data, display);
}


/*!
 * \brief elimina el mdulo de memoria
 * \param data mdulo principal
 */
void cursor_free (Cursor * data)
{
	SDL_FreeSurface (data->image);
}

/*!
 * \brief intenta seleccionar el personaje actual
 * \param data mdulo principal
 */
void cursor_select (Cursor * data)
{
	Cursor * other_cursor;
	Select_vs * select = (Select_vs *) data->world->scene;

 	/* si est en modo simple (1 jugador) ingresa al partido */	
	if (! select->vs_mode)
	{
		data->active = FALSE;
		return;
	}
	
	if (data->player_number)
		other_cursor = select->cursor_1;
	else
		other_cursor = select->cursor_2;

	/* evita que ambos cursores puedan elegir al mismo personaje */
	if (! other_cursor->active && other_cursor->index == data->index)
		return;

	data->active = FALSE;
}

/*!
 * \brief retorna una referencia al jugador seleccionado
 * \param data mdulo principal
 * \return el jugador seleccionado
 */
Player * cursor_get_player (Cursor * data)
{
	return data->player;
}


/*!
 * \brief Actualiza la referencia al jugador seleccionado
 * \param data mdulo principal
 */ 
void cursor_change_player (Cursor * data)
{
	data->player = characters_get_player (data->world->characters, 
			data->index);
}


/*!
 * \brief Imprime el nombre del personaje en pantalla
 * \param data mdulo principal
 * \param display gestor de impresin 
 */ 
void cursor_draw_player_nick (Cursor * data, Display * display)
{
	enum align align;
	int x;

	if (data->player_number)
	{
		x = 600;
		align = ALIGN_RIGHT;
	}
	else
	{
		x = 40;
		align = ALIGN_LEFT;
	}

	font_printf_align (data->world->font, display, x, 420, FALSE, align, 
			data->player->nick);
}


/*!
 * \brief informa si el cursor ha realizado la seleccin
 * \param mdulo principal
 * \return TRUE en caso de estar seleccionado
 */
int cursor_selected (Cursor * data)
{
	return (! data->active);
}
