/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "display.h"


/*!
 * \brief Genera el mdulo completo e inicializa la ventana principal
 * \param fullscreen indica si el juego comienza en pantalla completa
 * \return el mdulo generado
 */
Display * display_init (int fullscreen)
{
	Display * data;

	data = (Display *) malloc (sizeof (Display));

	if (! data)
	{
		util_print_fail_module ("Display");
		exit (1);
	}

	if (display_init_surfaces (data, fullscreen))
		exit (1);

	data->current_size = 0;
	data->last_size = 0;
	data->update_entire = FALSE;

	memset (data->current, 0x0, sizeof (SDL_Rect) * 256);
	memset (data->last, 0x0, sizeof (SDL_Rect) * 256);
	
	return data;
}

/*!
 * \brief Genera las superficies almacenadas en el mdulo
 * \param data mdulo principal
 * \param fullscreen indica si debe iniciar en pantalla completa
 * \return 1 en caso de error, 0 en otro caso
 */
int display_init_surfaces (Display * data, int fullscreen)
{
	Uint32 flags = SDL_SWSURFACE;

	if (fullscreen)
		flags |= SDL_FULLSCREEN;
	
	data->screen = SDL_SetVideoMode (640, 480, 16, flags);

	if (! data->screen)
	{
		printf (_("Error: %s\n"), SDL_GetError ());
		return 1;
	}

	data->background = SDL_DisplayFormat (data->screen);

	if (! data->background)
	{
		printf (_("Error: Can't create the background image\n"));
		return 1;
	}
	
	SDL_WM_SetCaption ("Pocket Volleyball (" PACKAGE_STRING ")", NULL);
	SDL_ShowCursor (SDL_DISABLE);

	return 0;

}

/*!
 * \brief Muestra los cmbios en la pantalla
 * \param data mdulo principal
 */
void display_update (Display * data)
{
	SDL_Rect all [512];
	int all_size;

	if (data->update_entire)
	{
		SDL_UpdateRect (data->screen, 0, 0, 0, 0);
		data->update_entire = FALSE;
		display_dirty_restore (data);
		return;
	}

	all_size = display_dirty_extract_all (data, all);

	SDL_UpdateRects (data->screen, all_size, all);
	display_dirty_clear (data);
	display_dirty_restore (data);
}

/*!
 * \brief Aade un rectngulo a la estructura
 * \param data mdulo principal
 */
void display_add (Display * data, SDL_Rect * rect)
{
	int i;

	/* Evita duplicados exctos */
	
	for (i = 0; i < data->current_size; i ++)
	{
		if (display_equal_rect (data->current [i], (* rect)))
			return;
	}

	data->current [data->current_size] = * rect;
	data->current_size ++;
}

/*!
 * \brief restaura el fondo de pantalla en todas las zonas modificadas
 * \param data mdulo principal
 */
void display_dirty_restore (Display * data)
{
	int i;
	
	for (i = 0; i < data->last_size; i ++)
	{
		SDL_BlitSurface (data->background, data->last + i, 
				data->screen, data->last + i);
	}
}

/*!
 * \brief Copia todos los rectngulos a rect
 * \param data mdulo principal
 * \param rect almacenar todos los rectngulos
 * \return cantidad de rectngulos copiados
 */
int display_dirty_extract_all (Display * data, SDL_Rect * rect)
{
	int elements = 0;
	int i;

	for (i = 0; i < data->last_size; i ++)
		display_dirty_add_rectangle (rect, & elements, data->last + i);

	for (i = 0; i < data->current_size; i ++)
		display_dirty_add_rectangle (rect, & elements, data->current+i);

	return elements;
}

/*!
 * \brief Elimina los rectngulos anteriores y libera los actuales
 * \param data mdulo principal
 */
void display_dirty_clear (Display * data)
{
	int i;

	for (i = 0; i < data->current_size; i ++)
		data->last [i] = data->current [i];

	data->last_size = i;
	data->current_size = 0;
}

/*!
 * \brief Informa si el juego se encuentra en pantalla completa
 * \param data mdulo principal
 * \return 1 en caso de estr en pantalla completa, 0 en otro caso
 */
int display_get_fullscreen (Display * data)
{
	if (data->screen->flags && SDL_FULLSCREEN)
		return TRUE;
	else
		return FALSE;
}

/*!
 * \brief Habilita el modo de pantalla completa
 * \param data mdulo principal
 */
void display_enable_fullscreen (Display * data)
{
	if (! display_get_fullscreen (data))
	{
		int flags = data->screen->flags;

		flags |= SDL_FULLSCREEN;

		data->screen = SDL_SetVideoMode (640, 480, 16, flags);

		if (!data->screen)
		{
			util_print_fail_module ("Display");
			exit (1);
		}
		
		SDL_BlitSurface (data->background, NULL, data->screen, NULL);
		SDL_Flip (data->screen);
	}
}

/*!
 * \brief Deshablita el modo de pantalla completa, pasa a modo ventana
 * \param data mdulo principal
 */
void display_disable_fullscreen (Display * data)
{
	if (display_get_fullscreen (data))
	{
		int flags = data->screen->flags;

		flags ^= SDL_FULLSCREEN;

		data->screen = SDL_SetVideoMode (640, 480, 16, flags);

		if (!data->screen)
		{
			util_print_fail_module ("Display");
			exit (1);
		}

		SDL_BlitSurface (data->background, NULL, data->screen, NULL);
		SDL_Flip (data->screen);
	}
}

/*!
 * \brief Elimina el mdulo por completo
 * \param data mdulo principal
 */
void display_free (Display * data)
{
	SDL_FreeSurface (data->background);
	free (data);
}

/*!
 * \brief Define el fondo de pantalla, para que Display genere una copia propia
 * \param data mdulo principal
 * \param background superficie a copiar, deber liberarla luego de llamar a esta funcin, ya que solo se copiar su contenido
 */
void display_set_background (Display * data, SDL_Surface * background)
{
	SDL_BlitSurface (background, NULL, data->background, NULL);
	SDL_BlitSurface (background, NULL, data->screen, NULL);
	data->update_entire = TRUE;
}


/*!
 * \brief Imprime un cuadro de la grilla sobre Display
 * \param data mdulo principal
 * \param src superficie origen
 * \param frame indice del cuadro a imprimir
 * \param columns cantidad de columnas de la grilla
 * \param x coordenada horizontal
 * \param y coordenada verticial
 * \param flip indica si la imagen se debe invetir de manera horizontal
 */
void display_draw (Display * data, SDL_Surface * src, int frame, int columns, 
		int x, int y, int flip)
{
	int i;
	int w = src->w / columns;
	SDL_Rect src_rect = {frame * w, 0, w, src->h};
	SDL_Rect rect = {x, y, 0, 0};
	
	if (frame < 0 || frame > columns)
	{
		printf (_("Can't draw frame %d\n"), frame);
		return;
	}
	
	if (flip)
	{
		for (i = 0; i < src_rect.w; i ++)
		{
			SDL_Rect dst = {rect.x + src_rect.w - i, rect.y, 0, 0};
			SDL_Rect rsrc = {src_rect.x + i, src_rect.y, 
				1, src_rect.h};
			
			SDL_BlitSurface (src, & rsrc, data->screen, & dst);
		}
		
		rect.w = src_rect.w + 1;
		rect.h = src_rect.h;

		if (rect.x < 0)
		{
			rect.w = rect.w + rect.x;
			rect.x = 0;
		}

		/* evita que el rectngulo exceda el rea de pantalla */

		if (rect.x + rect.w > 640)
			rect.w = 640 - rect.x;

		if (rect.y + rect.h > 480)
			rect.h = 480 - rect.y;

		if (rect.x < 0)
		{
			rect.w += rect.x;
			rect.x = 0;
		}

		if (rect.y < 0)
		{
			rect.h += rect.y;
			rect.y = 0;
		}
	}
	else
	{
		SDL_BlitSurface (src, & src_rect, data->screen, & rect);
	}
	
	display_add (data, & rect);
}

/*!
 * \brief Imprime un cuadro de la grilla proyectado como una sombra en Display
 * \param data mdulo principal
 * \param src superficie origen
 * \param frame indice del cuadro a imprimir
 * \param columns cantidad de columnas de la grilla
 * \param x coordenada horizontal
 * \param y coordenada verticial
 * \param flip indica si la imagen se debe invetir de manera horizontal
 */
void display_draw_shadow (Display * data, SDL_Surface * src, int frame, 
		int columns, int x, int y, int flip)
{
	int i;
	int w = src->w / columns;
	SDL_Rect src_rect = {frame * w, 0, w, src->h};
	SDL_Rect rect = {x - 10, y, 0, 0};
	int dx;
	int dy;
	
	if (frame < 0 || frame > columns)
	{
		printf (_("Can't draw frame %d\n"), frame);
		return;
	}
	
	if (flip)
	{
		for (i = 0; i < src_rect.w; i ++)
		{
			SDL_Rect dst = {rect.x + src_rect.w - i, rect.y, 0, 0};
			SDL_Rect rsrc = {src_rect.x + i, src_rect.y, 
				1, src_rect.h};
			
			SDL_BlitSurface (src, & rsrc, data->screen, & dst);
		}
		
		rect.w = src_rect.w + 1;
		rect.h = src_rect.h;

		if (rect.x < 0)
		{
			rect.w = rect.w + rect.x;
			rect.x = 0;
		}

	}
	else
	{
		Uint32 pixel;

		for (dx = src_rect.x; dx < src_rect.x + w; dx ++)
		{
			for (dy = 0; dy < src_rect.h; dy ++)
			{
				pixel = util_get_pixel (src, dx, dy);

				util_put_pixel (data->screen, rect.x, rect.y,
						pixel);

				rect.y ++;
			}

			rect.x ++;
		}


		rect.w = src_rect.w;
		rect.h = src_rect.h;
	}
	
	/* evita que el rectngulo exceda el rea de pantalla */

	if (rect.x + rect.w > 640)
		rect.w = 640 - rect.x;

	if (rect.y + rect.h > 480)
		rect.h = 480 - rect.y;

	if (rect.x < 0)
	{
		rect.w += rect.x;
		rect.x = 0;
	}

	if (rect.y < 0)
	{
		rect.h += rect.y;
		rect.y = 0;
	}

	display_add (data, & rect);
}




/*!
 * \brief Aade un rectngulo a un vector de rectngulos evitando duplicados
 * \param rect puntero al primer elemento del vector
 * \param elements cantidad de elementos del vector
 * \param new puntero al rectngulo que se debe aadir
 */
void display_dirty_add_rectangle (SDL_Rect * rect, int * n, SDL_Rect * new)
{
	int i;

	for (i = 0; i < * n; i ++)
	{
		if (util_compare_two_rects ( (rect + i), new))
			return;
	}

	rect [i] = * new;
	* n += 1;
}


/*!
 * \brief Imprime de manera similar a SDL_BlitSurface 
 * \param data mdulo principal
 * \param src superficie origen
 * \param rect rectngulo origen
 * \param x coordenada horizontal
 * \param y coordenada verticial
 * \param flip indica si la imagen se debe invetir de manera horizontal
 */
void display_blit (Display * data, SDL_Surface * src, SDL_Rect * src_rect,
		int x, int y, int flip)
{
	int i;
	SDL_Rect rect = {x, y, 0, 0};
	
	if (flip)
	{
		for (i = 0; i < src_rect->w; i ++)
		{
			SDL_Rect dst = {rect.x + src_rect->w - i, rect.y, 0, 0};
			SDL_Rect rsrc = {src_rect->x + i, src_rect->y, 
				1, src_rect->h};
			
			SDL_BlitSurface (src, & rsrc, data->screen, & dst);
		}
		
		rect.w = src_rect->w + 1;
		rect.h = src_rect->h;

		if (rect.x < 0)
		{
			rect.w = rect.w + rect.x;
			rect.x = 0;
		}

		/* evita que el rectngulo exceda el rea de pantalla */

		if (rect.x + rect.w > 640)
			rect.w = 640 - rect.x;

		if (rect.y + rect.h > 480)
			rect.h = 480 - rect.y;

		if (rect.x < 0)
		{
			rect.w += rect.x;
			rect.x = 0;
		}

		if (rect.y < 0)
		{
			rect.h += rect.y;
			rect.y = 0;
		}
	}
	else
	{
		SDL_BlitSurface (src, src_rect, data->screen, & rect);
	}

	display_add (data, & rect);

}


/*!
 * \brief Imprime un punto grande en la pantalla, til para depurar el programa
 * \param data mdulo principal
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \param color color del punto
 */
void display_put_big_pixel (Display *data, int x, int y)
{
	SDL_Rect rect = {x, y, 2, 2};

	SDL_FillRect (data->screen, &rect, 0);

	display_add (data, & rect);
}





