/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef _DISPLAY_H
#define _DISPLAY_H

/*!
 * \file
 */

#include "common.h"

/*!
 * \brief Gestiona la pantalla de la aplicacin.
 *
 * Dirty gestiona una tcnica de impresin llamada Dirty Rectangles, en 
 * donde se procura imprimir solo aquellas partes de la pantalla que 
 * varan de un cuadro a otro. La tcnica de impresin mas utilizada en la
 * biblioteca SDL es Double Buffer o Page Fliping, donde se acualiza la
 * pantalla completa mediante llamados a SDL_Flip o SDL_UpdateRect (con
 * valores 0, 0, 0, 0).
 *
 * Para que una escena pueda operar esta estructura deber indicarle una
 * imgen de fondo a Display llamando a 'display_set_background'. Se debe 
 * llamar a esa funcin solo una vez, cuando inicia la escena.
 * 
 * Luego, por cada grfico a imprimir se deber llamar a la funcin 
 * 'display_draw' con una referencia a Display. La funcin 'display_draw'
 * accede directamente a la superficie 'screen' y luego llama a la
 * funcin 'display_dirty_add' para indicarle que zona de la pantalla ha 
 * modificado.
 *
 * El resto de las funciones pblicas (las ms importantes) son llamadas por
 * el mdulo `World` desde su funcin 'main_loop'
 */
typedef struct display_t
{
	SDL_Surface * screen;		/*!< Superficie principal */
	SDL_Surface * background;	/*!< Fondo de la escena */

	int current_size;		/*!< Elementos del vector current */
	int last_size;			/*!< Elementos del vector last */
	int update_entire;		/*!<  Actualizar toda la pantalla ? */
	
	SDL_Rect current [256];		/*!< Rectngulos actuales */
	SDL_Rect last [256];		/*!< Rectngulos del cuadro anterior */
} Display;

/*!
 * \brief Verifica si 2 rectngulos son iguales
 */
#define display_equal_rect(X,Y) (X.x == Y.x)? (X.y == Y.y)? (X.w == Y.w)? (X.h == Y.h)? 1 : 0 : 0 : 0 : 0

Display * display_init (int fullscreen);
int display_init_surfaces (Display * data, int fullscreen);
void display_update (Display * data);
void display_free (Display * data);
void display_dirty_restore (Display * data);
void display_dirty_clear (Display * data);
int display_dirty_extract_all (Display * data, SDL_Rect * rect);
void display_set_background (Display * data, SDL_Surface * background);

int display_get_fullscreen (Display * data);
void display_enable_fullscreen (Display * data);
void display_disable_fullscreen (Display * data);
void display_dirty_add_rectangle (SDL_Rect * rect, int * n, SDL_Rect * new);

void display_draw (Display * data, SDL_Surface * src, int frame, int columns, 
		int x, int y, int flip);

void display_draw_shadow (Display * data, SDL_Surface * src, int frame, 
		int columns, int x, int y, int flip);


void display_blit (Display * data, SDL_Surface * src, SDL_Rect * rect, 
		int x, int y, int flip);

void display_put_big_pixel (Display *data, int x, int y);
#endif
