/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "font.h"
#include <stdarg.h>

/*!
 * \brief Genera el mdulo Font
 * \param file ruta de la grilla que contiene todos los caracteres
 * \return el mdulo creado
 */
Font * font_init (char * file, char * file_selected, char * file_disabled)
{
	Font * data;

	data = (Font *) malloc (sizeof (Font));

	if (! data)
	{
		util_print_fail_module ("Font");
		exit (1);
	}

	data->image = util_load_image (file, TRUE);

	if (data->image == NULL)
	{
		printf (_("Can't load '%s' image\n"), file);
		util_print_fail_module ("Font");
		free (data);
		exit (1);
	}

	data->image_selected = util_load_image (file_selected, TRUE);

	if (data->image_selected == NULL)
	{
		printf (_("Can't load '%s' image\n"), file_selected);
		util_print_fail_module ("Font");
		free (data);
		exit (1);
	}

	data->image_disabled = util_load_image (file_disabled, TRUE);

	if (data->image_disabled == NULL)
	{
		printf (_("Can't load '%s' image\n"), file_disabled);
		util_print_fail_module ("Font");
		free (data);
		exit (1);
	}

	font_scan_rects (data);
	
	return data;
}

/*!
 * \brief imprime un caracter sobre la pantalla principal
 * \param data mdulo principal
 * \param display mdulo que controla la pantalla
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \param type tipo de texto a imprimir
 * \param c caracter a imprimir
 * \return el ancho del caracter impreso
 */
int font_print_char (Font * data, Display * display, int x, int y, 
		enum type type, char c)
{
	int i;
	SDL_Surface * image;

	switch (type)
	{
		case TYPE_NORMAL:
			image = data->image;
			break;

		case TYPE_SELECTED:
			image = data->image_selected;
			break;

		case TYPE_DISABLED:
			image = data->image_disabled;
			break;

		default:
			/* nunca debera ocurrir */
			image = NULL;
			break;
	}

	
	/* caso particular, los espacios no se imprimen y 
	 * tienen el mismo ancho que el primer caracter */
	if (c == ' ')
		return data->rects [0].w;

	i = font_get_index (c);

	if (i > -1)
	{
		SDL_Rect rect;

		rect = data->rects [i];
		display_blit (display, image, & rect, x, y, 0);

		return rect.w;
	}

	return 0;
}

/*!
 * \brief Destruye el mdulo Font
 * \param data mdulo principal
 */
void font_free (Font * data)
{
	SDL_FreeSurface (data->image);
	SDL_FreeSurface (data->image_selected);
	SDL_FreeSurface (data->image_disabled);
	free (data);
}


/*!
 * \brief Busca el ndice que le corresponde a un caracter
 * \param c caracter a buscar
 * \return indice del caracter o -1 en caso de error
 */
int font_get_index (char c)
{
	int i;
	static char reference [] = "abcdefghijklmnopqrstuvwxyz"\
				   "ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
				   "1234567890" \
				   ""\
				   "!?@#$%&'+-=><*/,.:;-_()[]{}|^`~\\"\
				   "";

	for (i = 0; reference [i]; i ++)
	{
		if (c == reference [i])
			return i;
	}

	printf (_("Can't find '%c' letter in the font image.\n"), c);
	return -1;
}


/*!
 * \brief Imprime una cadena con formato al igual que la funcin printf
 * \param data mdulo principal
 * \param display mdulo que controla la pantalla
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \param type tipo de fuente a utilizar
 * \param format formato de la cadena
 */
void font_printf (Font * data, Display * display, int x, int y, enum type type,
		const char * format, ...)
{
	va_list va;
	char buffer [1024];
	int i;
	int dx = 0;

	va_start (va, format);
	vsprintf (buffer, format, va);
	
	for (i = 0; buffer [i]; i ++)
	{
		dx += font_print_char (data, display, dx + x, y, type, 
				buffer [i]);
	}
	
	va_end (va);
}


/*!
 * \brief Imprime una cadena con formato al igual que la funcin printf
 * \param data mdulo principal
 * \param display mdulo que controla la pantalla
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \param selected si debe resaltar el texto
 * \param format formato de la cadena
 */
void font_printf_align (Font * data, Display * display, int x, int y, 
		int selected, enum align align, const char * format, ...)
{
	va_list va;
	char buffer [1024];
	int text_width;

	va_start (va, format);
	vsprintf (buffer, format, va);

	text_width = font_get_text_width (data, buffer);

	switch (align)
	{
		case ALIGN_LEFT:
			font_printf (data, display, x, y, selected, buffer);
			break;

		case ALIGN_CENTER:
			font_printf (data, display, x - text_width / 2, y, 
					selected, buffer);
			break;

		case ALIGN_RIGHT:
			font_printf (data, display, x - text_width, y, 
					selected, buffer);
			break;
	}

	va_end (va);
}


/*!
 * \brief Analiza la imagen para encontrar la posicion de cada caracter
 * \param data mdulo principal
 */
void font_scan_rects (Font * data)
{
	int index = 0;
	int empty = 1;
	int w = 0;
	int i;
	SDL_Surface * ima = data->image;

	for (i = 0; i < ima->w; i ++)
	{
		if (font_is_empty_column (ima, i))
		{
			/* si estaba leyendo una letra entonces encontr 
			 * el ancho de la misma */
			if (! empty)
			{
				font_store_rect (data->rects + index, \
						i - w, 0, w, ima->h);
				empty = 1;
				index ++;
			}
		}
		else
		{
			/* si est recorriendo una letra aumenta el contador
			 * de ancho, en caso contrario encontr donde comienza
			 * la siguiente letra
			 */
			if (! empty)
				w ++;
			else
			{
				w = 1;
				empty = 0;
			}
		}
	}
}

/*!
 * \brief Informa si la columna (parmetro 2) es una linea completa de pixeles
 * transparentes en la superficie ima (parmetro 1)
 *
 * Se asume que el color transparente es aquel que coincide con el pixel
 * situado en la posicion (0,0) de la superficie.
 *
 * \param ima superficie para analizar
 * \param col columna a inspeccionar
 * \return 1 si est completamente compuesta de pxeles tranparentes
 */
int font_is_empty_column (SDL_Surface * ima, int col)
{
	Uint32 transparent_pixel = util_get_pixel (ima, 0, 0);
	int fila;
	
	/* busca un pixel opaco */
	for (fila = 0; fila < ima->h; fila ++)
	{
		if (transparent_pixel != util_get_pixel (ima, col, fila))
			return 0;
	}

	return 1;
}


/*!
 * \brief Define la dimensin del rectngulo rect (parmetro 1)
 * \param rect rectngulo destino
 * \param x posicin horizontal
 * \param y posicin vertical
 * \param w ancho
 * \param h algo
 */
void font_store_rect (SDL_Rect * rect, int x, int y, int w, int h)
{
	rect->x = x;
	rect->y = y;
	rect->w = w;
	rect->h = h;
}


/*!
 * \brief Informa la altura de cada fuente
 * \param data mdulo principal
 * \return la altura de cada fuente
 */
int font_get_h (Font * data)
{
	return data->image->h;
}


/*!
 * \brief Informa la longitud horizontal de una cadena de texto completa
 * \param data mdulo principal
 * \param text cadena
 * \return longitud de la cadena
 */
int font_get_text_width (Font * data, const char * text)
{
	int i;
	int width = 0;

	for (i = 0; text [i]; text ++)
		width += font_get_char_width (data, text [i]);

	return width;
}


/*!
 * \brief Informa la longitud horizontal de un caracter
 * \param data mdulo principal
 * \param c caracter
 * \return longitud horizontal del caracter
 */ 
int font_get_char_width (Font * data, char c)
{
	int i;

	if (c == ' ')
		return data->rects [0].w;

	i = font_get_index (c);

	if (i > -1)
		return data->rects [i].w;
	else
		return 0;
}
