/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "fps.h"

/*!
 * \brief Inicia el mdulo
 * \param fps_requested cantidad de cuadros por segundos solicidatos (optimo)
 */
Fps * fps_init (int fps_requested, int debug)
{
	Fps * data;

	data = (Fps *) malloc (sizeof (Fps));

	if (! data)
	{
		util_print_fail_module ("Fps");
		exit (1);
	}
	
	data->current_tick = SDL_GetTicks ();
	data->last_tick = data->current_tick;
	data->tick_last_fps_notify = data->current_tick;
	data->debug = debug;

	data->fps = 0;
	data->frequency = 1000 / fps_requested;
	data->max_frame_skip = 1;

	if (debug)
		fps_set_debug_title (data);
	
	return data;
}


/*!
 * \brief asegura que el juego mantenga la frecuencia de ejecucin solicitada
 *
 * Si es necesario, la funcin detiene al programa, utilizando SDL_Delay. En
 * cambio, si el equipo es muy lento, esta funcin indicar cuantos cuadros
 * lgicos deberan ejecutarse para compenzar la prdida de velocidad 
 * (equivalente a los `saltos` de cuadros)
 *
 * \param data mdulo principal
 * \return cantidad de ciclos lgicos que se deben ejecutar o 0 si se detuvo
 */
float fps_synchronize (Fps * data)
{
	static int delta;
	static int return_value = 0;
	
	data->current_tick = SDL_GetTicks ();

	delta = data->current_tick - data->last_tick;

	fps_update_display (data);

	if (delta >= data->frequency)
	{
		return_value = delta / data->frequency;

		if (return_value > data->max_frame_skip)
			return_value = data->max_frame_skip;

		data->last_tick += return_value * data->frequency;
		
		data->fps ++;

		return return_value;
	}
	else
	{
		SDL_Delay (1);
		return 0;
	}
}


/*!
 * \brief Termina con el mdulo
 * \param data mdulo principal
 */
void fps_free (Fps * data)
{
	free (data);
}

/*!
 * \brief Reinicia los contadores de tiempo. Resulta de utilidad para 'evitar' 
 * un salto de cuadros muy pronunciado cuando se genera o cambia una escena.
 * \param data mdulo principal
 */
void fps_reset (Fps * data)
{
	data->current_tick = data->last_tick = SDL_GetTicks ();
}

/*!
 * \brief Actualiza el contador de cuadros e intenta mostrarlo en pantalla
 * \param data mdulo principal
 */
void fps_update_display (Fps * data)
{
	if (data->debug)
	{
		if (data->current_tick - data->tick_last_fps_notify > 1000)
		{
			fps_set_debug_title (data);

			data->fps = 0;
			data->tick_last_fps_notify += 1000;
		}
	}
}


void fps_set_debug_title (Fps * data)
{
	char title [200];
	sprintf (title, "Volleyball at %d fps", data->fps);

	SDL_WM_SetCaption (title, NULL);
}
