/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "item.h"
#include "common.h"

/*!
 * \brief Genera un nuevo mdulo item
 * \param title titulo de la opcin
 * \param x posicin horizontal
 * \param y posicin vertical
 * \param font mdulo de fuentes
 * \param display mdulo utilizado para imprimir grficos
 * \return el mdulo creado
 */
Item * item_new (char * title, int x, int y, Font * font, Display * display)
{
	Item * data;

	data = (Item *) malloc (sizeof (Item));

	if (!data)
	{
		util_print_fail_module ("Item");
		exit (1);
	}

	strcpy (data->title, title);
	data->x = x;
	data->y = y;
	data->font = font;
	data->display = display;
	data->type = TYPE_NORMAL;
	data->enabled = TRUE;
	data->selected = FALSE;

	/* Slo para el mdulo `item_progress` */
	data->function_to_notify = NULL;
	data->module_receiver = NULL;

	return data;
}

/*!
 * \brief Genera un item pero que cuenta con un progreso (o volmen)
 * \param title titulo de la opcin
 * \param x posicin horizontal
 * \param y posicin vertical
 * \param font mdulo de fuentes
 * \param display mdulo utilizado para imprimir grficos
 * \param max_progress mximo valor permitido para el desplazador
 * \param initial_progress valor inicial del desplazador
 * \return el mdulo creado
 */
Item * item_progress_new (char * title, int x, int y, Font * font, 
		Display * display, int max_progress, int initial_progress)
{
	Item * data = item_new (title, x, y, font, display);

	data->max_progress = max_progress;
	data->progress_value = initial_progress;
	data->type = ITEM_PROGRESS_TYPE;
	data->function_to_notify = NULL;
	data->module_receiver = NULL;
	
	return data;
}



/*!
 * \brief imprime la opcin en pantalla
 * \param data mdulo principal
 */
void item_draw (Item * data)
{
	enum type type = item_get_font_type (data);

	font_printf (data->font, data->display, data->x, data->y, type, 
			data->title);
		
	if (data->type == ITEM_PROGRESS_TYPE)
		item_progress_draw_value (data);

}

/*!
 * \brief selecciona un item del men
 * \param data mdulo principal
 */
void item_enter (Item * data)
{
	data->selected = TRUE;
}

/*!
 * \brief quita la seleccin de un item de men
 * \param data mdulo principal
 */
void item_leave (Item * data)
{
	data->selected = FALSE;
}


/*!
 * \brief Deshabilita el ttulo de la opcin
 * \param data mdulo principal
 */
void item_disable (Item * data)
{
	data->enabled = FALSE;
}

/*!
 * \brief Habilita el ttulo del men
 * \param data mdulo principal
 */
void item_enable (Item * data)
{
	data->enabled = TRUE;
}


/*!
 * \brief informa si se puede seleccionar el ttulo de la opcin
 * \param data mdulo principal
 */
int item_is_enabled (Item * data)
{
	return (data->enabled == TRUE);
}

/*!
 * \brief informa la posicin vertical del item
 * \param mdulo principal
 * \return posicin vertical
 */
int item_get_y (Item * data)
{
	return data->y;
}

/*!
 * \brief elimina el mdulo
 * \param data mdulo principal
 */
void item_free (void * data)
{
	Item * tmp;

	tmp = (Item *) data;
	free (tmp);
}


/*!
 * \brief Aumenta el valor de la barra de desplazamiento
 * \param data mdulo principal
 */
void item_progress_add (Item * data)
{
	if (data->type == ITEM_PROGRESS_TYPE)
	{
		data->progress_value ++;

		if (data->progress_value > data->max_progress - 1)
			data->progress_value = data->max_progress - 1;
	}

	if (data->module_receiver)
		data->function_to_notify (data->module_receiver, 
				(data->progress_value * 128) / 19);
}

/*!
 * \brief Reduce el valor de la barra de desplazamiento
 * \param data mdulo principal
 */
void item_progress_subtract (Item * data)
{
	if (data->type == ITEM_PROGRESS_TYPE)
	{
		data->progress_value --;

		if (data->progress_value < 0)
			data->progress_value = 0;
	}

	if (data->module_receiver)
		data->function_to_notify (data->module_receiver,
				(data->progress_value * 128) / 19);
}


/*!
 * \brief Muestra la barra de progreso asociada al Item
 * \param data mdulo principal
 */
void item_progress_draw_value (Item * data)
{
	int i;
	char tmp [1024];
	enum type type = item_get_font_type (data);
	int initial_x = data->x + font_get_char_width (data->font, ' ') + 
		font_get_text_width (data->font, data->title);
	
	for (i = 0; i < data->progress_value; i ++)
		tmp [i] = '|';
	
	tmp [i] = '\0';
	
	font_printf (data->font, data->display, initial_x, data->y, type, tmp);
}


enum type item_get_font_type (Item * data)
{
	if (! data->enabled)
		return TYPE_DISABLED;
	else
	{
		if (data->selected)
			return TYPE_SELECTED;
		else
			return TYPE_NORMAL;
	}

}

void item_on_change_progress_value (Item * data, 
		void (* function) (void * module, int value), 
		void * module)
{
	data->function_to_notify = function;
	data->module_receiver = module;
}
