/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "common.h"
#include "menu.h"
#include "item.h"
#include "control.h"

/*!
 * \brief inicia el mdulo Menu
 * \param world mdulo World
 * \param initial_option opcin seleccionada por defecto
 * \return el mdulo creado
 */
Menu * menu_init (World * world)
{
	Menu * data;
	
	data = (Menu *) malloc (sizeof (Menu));

	if (! data)
	{
		util_print_fail_module ("Menu");
		exit (1);
	}

	data->option = 0;
	data->world = world;
	data->items_n = 0;
	data->start_y = 200;

	data->pointer = pointer_new (80, 1, 1, world->display);
	
	return data;
}

/*!
 * \brief imprime el men en pantalla
 * \param data mdulo principal
 */
void menu_draw (Menu * data)
{
	int i;

	for (i = 0; i < data->items_n; i ++)
		item_draw (data->items [i]);
	
	pointer_draw (data->pointer);
}

/*!
 * \brief actualiza la posicion del cursor y captura una seleccin
 * \param data mdulo principal
 * \return >= 0 si seleccion una opcin, el valor indica el ndice seleccionado. En caso de no seleccionar una opcin retornar -1.
 */
int menu_update (Menu * data)
{
	Control * control = data->world->control;

	pointer_update (data->pointer);

	if (control_query_no_repeat (control, ENTER) ||
		control_query_no_repeat (control, P1_SELECT) ||
		control_query_no_repeat (control, P2_SELECT))
	{
		audio_play_sound (data->world->audio, SND_SELECT_OPTION);
		return data->option;
	}

	if (control_query_no_repeat (control, P1_DOWN) || 
			control_query_no_repeat (control, P2_DOWN))
		menu_change_option (data, + 1);

	if (control_query_no_repeat (control, P1_UP) ||
			control_query_no_repeat (control, P2_UP))
		menu_change_option (data, - 1);

	if (control_query (control, P1_RIGHT) || 
			control_query (control, P2_RIGHT))
	{
		item_progress_add (data->items [data->option]);
	}

	if (control_query (control, P1_LEFT) ||
			control_query (control, P2_LEFT))
	{
		item_progress_subtract (data->items [data->option]);
	}


	
	return -1;
}

/*!
 * \brief intercambia la opcin seleccionada
 * \param data mdulo principal
 * \param delta desplazamiento del cursor (generalmente valdr 1 o -1)
 */
void menu_change_option (Menu * data, int delta)
{
	if (item_is_enabled (data->items [data->option]))
		item_leave (data->items [data->option]);
	
	data->option += delta;

	if (data->option < 0)
		data->option = data->items_n - 1;
	else
	{
		if (data->option >= data->items_n)
			data->option = 0;
	}

	if (item_is_enabled (data->items [data->option]))
	{
		pointer_change (data->pointer, 
				item_get_y (data->items [data->option]));
		item_enter (data->items [data->option]);
		audio_play_sound (data->world->audio, SND_CHANGE_OPTION);
	}
	else
		menu_change_option (data, delta);
}

/*!
 * \brief elimina el mdulo
 * \param data mdulo principal
 */
void menu_free (Menu * data)
{
	int i;

	for (i = 0; i < data->items_n; i ++)
		item_free (data->items [i]);

	pointer_free (data->pointer);
	free (data);
}


/*!
 * \brief Aade una opcin al men
 * \param data mdulo pricipal
 * \param title ttulo de la opcin 
 */
void menu_add_item (Menu * data, char * title)
{
	data->items [data->items_n] = item_new (title, 100, data->start_y, 
			data->world->font, data->world->display);

	menu_internal_add_item (data);
}

void menu_add_progress_item (Menu * data, char * title, int max_progress, 
		int initial_progress)
{
	data->items [data->items_n] = item_progress_new (title, 100, 
			data->start_y, data->world->font, 
			data->world->display, max_progress, initial_progress);

	menu_internal_add_item (data);
}


void menu_internal_add_item (Menu * data)
{
	data->start_y += font_get_h (data->world->font);
	data->items_n ++;

	/* si es el primer elemento lo selecciona y 
	 * posiciona el puntero */
	if (data->items_n == 1)
	{
		pointer_set_y (data->pointer, item_get_y (data->items [0]));
		pointer_change (data->pointer, item_get_y (data->items [0]));
		item_enter (data->items [0]);
	}
}


/*!
 * \brief Deshabilita un tem del men
 * \param data mdulo principal
 * \param index ndice del tem
 */
void menu_disable_item (Menu * data, int index)
{
	item_disable (data->items [index]);

	if (data->option == index)
		menu_change_option (data, +1);
}


/*!
 * \brief Habilita un tem del men
 * \param data mdulo principal
 * \param index ndice del tem
 */
void menu_enable_item (Menu * data, int index)
{
	item_enable (data->items [index]);
}


void menu_set_initial_option (Menu * data, int initial_option)
{
	item_leave (data->items [data->option]);
	
	data->option = initial_option;

	pointer_set_y (data->pointer, item_get_y (data->items[initial_option]));
	item_enter (data->items [initial_option]);
}
