/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "player.h"
#include "player_ai.h"
#include "ball.h"


/*!
 * \brief asigna los estados que permiten al personaje tener AI
 * \param mdulo principal
 */
void player_set_ai_control (Player * data)
{
	/* see `player_ia.c` and `player_ia.h` */
	player_ai_do_start (data);
}

/*!
 * \brief define que control manejar los movimientos del personaje
 * \param data mdulo principal
 * \param control controlador
 */ 
void player_set_control (Player * data, Player_control * control)
{
	data->control = control;
	player_do_ready (data);
}



/*!
 * \brief Genera un nuevo personaje del juego en base a un directorio
 * \param path ruta hacia el directorio `characters`
 * \param file nombre del directorio que podra contener un personaje
 * \param debug indica si se deben imprimir las lineas guia
 * \return el mdulo creado o NULL en caso de error
 */
Player * player_init (char * path, char * file, int debug)
{
	Player * data;
	char complete_path [1024];

	printf ("\t%s:\t", file);
	sprintf (complete_path, "%s/%s", path, file);

	data = (Player *) malloc (sizeof (Player));

	if (! data)
	{
		util_print_fail_module ("Player");
		return NULL;
	}

	data->control = NULL;

	if (player_init_examine_files (complete_path))
	{
		free (data);
		return NULL;
	}

	if (player_init_load_images (data, complete_path))
	{
		free (data);
		return NULL;
	}


	if (player_init_load_attributes (data, complete_path))
	{
		free (data);
		return NULL;
	}

	if (debug)
		player_draw_debug_lines (data);
	
	printf (_("OK"));
	printf ("\n");

	data->debug = debug;
	player_do_ready (data);

	return data;
}


/*!
 * \brief Elimina el mdulo y libera memoria
 * \param data mdulo principal (bajo modelado a void - ver node_free_all)
 */
void player_free (void * _data)
{
	Player * data = (Player *) _data;

	animation_free (data->animation);

	SDL_FreeSurface (data->image);
	SDL_FreeSurface (data->preview);

	free (data);
}


/*!
 * \brief Verifica si existen los archivos del personaje
 * \param complete_path ruta hacia el directorio que contiene al personaje
 * \return 1 si presenta algn error, 0 en otro caso
 */
int player_init_examine_files (char * complete_path)
{
	if (! util_directory_exist (complete_path))
	{
		printf (_("Fail\n\t\t"));
		printf (_("This is not a directory\n"));
		return 1;
	}

	if (! util_file_exist (complete_path, "preview.png"))
	{
		printf (_("Fail\n\t\t"));
		printf (_("No such '%s' file\n"), "preview.png");
		return 1;
	}

	if (! util_file_exist (complete_path, "sprites.png"))
	{
		printf (_("Fail\n\t\t"));
		printf (_("No such '%s' file\n"), "sprites.png");
		return 1;
	}

	if (! util_file_exist (complete_path, "info.txt"))
	{
		printf (_("Fail\n\t\t"));
		printf (_("No such '%s' file\n"), "info.txt");
		return 1;
	}
	
	return 0;
}


/*!
 * \brief Carga las imgenes del personaje al mdulo
 * \param data mdulo principal
 * \param complete_path ruta completa hacia el directorio del personaje
 * \return 1 en caso de error, 0 en otro caso
 */
int player_init_load_images (Player * data, char * complete_path)
{
	char path_image [1024];
	char path_preview [1024];

	sprintf (path_image, "%s/%s", complete_path, "/sprites.png");
	sprintf (path_preview, "%s/%s", complete_path, "/preview.png");

	data->preview = util_load_image (path_preview, FALSE);

	if (data->preview == NULL)
	{
		printf (_("Fail\n\t\t"));
		printf (_("Can't load '%s' image\n"), path_preview);
		return 1;
	}

	data->image = util_load_image (path_image, FALSE);

	if (data->image == NULL)
	{
		printf (_("Fail\n\t\t"));
		printf (_("Can't load '%s' image\n"), path_image);
		return 1;
	}
	

	return 0;
}


/*!
 * \brief Carga el archivo de atributos `info.txt` al mdulo
 * \param data mdulo principal
 * \param complete_path ruta hacia el directorio del personaje
 * \return 1 en caso de error, 0 en otro caso
 */
int player_init_load_attributes (Player * data, char * complete_path)
{
	char file [1024];
	int n = MAX_OPTIONS;
	FileOptions options [MAX_OPTIONS] = {
		{"info_version", ""},
		{"nick", ""},
		{"description", ""},
		{"author", ""},
		{"email", ""},
		{"web", ""},
		{"frames", ""},
		{"hotspot_x", ""},
		{"hotspot_y", ""},
		{"collision_x", ""},
		{"collision_y", ""},
		{"ratio", ""},
		{"stand", ""},
		{"foward", ""},
		{"backward", ""},
		{"jump", ""},
		{"select", ""},
		{"touch_back", ""},
		{"touch_up", ""},
		{"touch_front", ""},
		{"delay", ""},
		{"touch_up_in_air", ""},
		{"touch_back_in_air", ""},
		{"touch_front_in_air", ""},
		{"jump_power", ""},
		{"width", ""},
		{"speed", ""}
	};

	sprintf (file, "%s/%s", complete_path, "info.txt");

	if (file_load (options, MAX_OPTIONS, file))
		return 1;

	sprintf (data->web, file_get_value (options, n, "web"));
	sprintf (data->nick, file_get_value (options, n, "nick"));
	sprintf (data->email, file_get_value (options, n, "email"));
	sprintf (data->author, file_get_value (options, n, "author"));
	sprintf (data->description, file_get_value (options, n, "description"));

	data->delay = atoi (file_get_value (options, n, "delay"));
	data->width = atoi (file_get_value (options, n, "width"));
	data->speed = atoi (file_get_value (options, n, "speed"));
	data->columns = atoi (file_get_value (options, n, "frames"));
	data->hotspot_x = atoi (file_get_value (options, n, "hotspot_x"));
	data->hotspot_y = atoi (file_get_value (options, n, "hotspot_y"));
	data->jump_power = atoi (file_get_value (options, n, "jump_power"));	
	data->collision_x = atoi (file_get_value (options, n, "collision_x"));
	data->collision_y = atoi (file_get_value (options, n, "collision_y"));
	data->collision_ratio = atoi (file_get_value (options, n, "ratio"));
	
	/* Animaciones */
	data->animation = animation_init (data->delay);
	
	animation_add (data->animation, READY, 
			file_get_value (options, n, "stand")); 
	animation_add (data->animation, RUN_LEFT, 
			file_get_value (options, n, "backward")); 
	animation_add (data->animation, RUN_RIGHT, 
			file_get_value (options, n, "foward")); 
	animation_add (data->animation, JUMP,
			file_get_value (options, n, "jump"));
	animation_add (data->animation, IN_SELECT,
			file_get_value (options, n, "select"));
	animation_add (data->animation, TOUCH_BACK, 
			file_get_value (options, n, "touch_back"));
	animation_add (data->animation, TOUCH_UP, 
			file_get_value (options, n, "touch_up"));
	animation_add (data->animation, TOUCH_FRONT, 
			file_get_value (options, n, "touch_front"));

	animation_add (data->animation, TOUCH_BACK_JUMPING, 
			file_get_value (options, n, "touch_back_in_air"));
	animation_add (data->animation, TOUCH_UP_JUMPING, 
			file_get_value (options, n, "touch_up_in_air"));
	animation_add (data->animation, TOUCH_FRONT_JUMPING, 
			file_get_value (options, n, "touch_front_in_air"));

	/*
	strcpy (data->info_version, file_get_value(options,n, "info_version"));
	*/

	return 0;
}


/*!
 * \brief Imprime lineas guias sobre las grilla de cuadros de animacin
 * \data mdulo principal
 */
void player_draw_debug_lines (Player * data)
{
	Uint32 color = SDL_MapRGB (data->image->format, 240, 220, 240);
	int i;
	int dx = 0;

	for (i = 0; i < data->columns; i ++)
	{
		util_draw_circle (data->image, color, dx + data->collision_x, 
				data->collision_y, data->collision_ratio);
		
		dx += data->image->w / data->columns;
		
		/*
		SDL_Rect tmp = {dx + data->hotspot_x - data->ancho / 2,
		(data->eje_y + data->colision_y) / 2,
		data->ancho, 3};
		
		imprimir_circulo (data->imagen, color,
		dx + data->colision_x, data->colision_y,
		data->colision_radio);
		
		imprimir_cruz (data->imagen, color,
		dx + data->eje_x,
		data->eje_y);
		
		imprimir_rectangulo (data->imagen, color, & tmp);
		
		dx += data->imagen->w / data->columnas;
		}
		*/
	}
					
}

/*!
 * \brief Imprime la vista preliminar del personaje (64x64 pxeles)
 * \param data mdulo principal
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \param display mdulo gestor de pantalla
 */
void player_draw_preview (Player * data, int x, int y, Display * display)
{
	display_draw (display, data->preview, 0, 1, x, y, FALSE);
}


/*!
 * \brief Imprime al personaje sobre la pantalla
 * \param data mdulo principal
 * \param display gestor de impresin en pantalla
 */
void player_draw (Player * data, Display * display)
{
	int dx = 0;
	int frame_width = data->image->w / data->columns;
	
	if (data->flip)
		dx = - frame_width + data->hotspot_x;
	else
		dx = -data->hotspot_x;

	display_draw (display, data->image, data->animation->actual_frame, 
			data->columns, data->x + dx, 
			data->y - data->hotspot_y, data->flip);
}


/*!
 * \brief Actualizacin lgica
 * \param data mdulo principal
 */ 
void player_update (Player * data)
{
	data->state (data);
}


/*!
 * \brief Se inicia un nuevo partido
 * \param data mdulo principal
 * \param ball pelota del juego
 * \param indice del jugador (0 o 1)
 * \param left limite izquiero del escenario que puede utilizar el jugador
 * \param rught limite derecho del escenario que puede utilizar el jugador
 */ 
void player_init_game (Player * data, struct ball * ball, int player, 
		int left, int right)
{
	data->ball = ball;

	if (player)
		player_set_position (data, 490, FLOOR, TRUE);
	else
		player_set_position (data, 150, FLOOR, FALSE);

	player_set_arena_boudary (data, left, right);
	data->initial_jump_speed = 0;
}


/*!
 * \brief Define una nueva posicin para el personaje
 * \param data mdulo principal
 * \param x coordenada horizontal de los pies
 * \param y coordenada vertical de los pies 
 * \param flip espejado horizontal
 */
void player_set_position (Player * data, int x, int y, int flip)
{
	data->flip = flip;
	data->y = y;
	data->x = x;
}


/*!
 * \brief Altera el estado actual del personaje
 * \param data mdulo principal
 * \param init funcin que inicia el estado
 * \param update funcin que actualiza el estado (se llamar constantemente)
 */
void player_change_state (Player * data, void (* init) (Player * data), 
		void (* update) (Player * data))
{
	if (init)
		init (data);

	data->state = update;
}


/*!
 * \brief Funcin que gestiona el estado READY (mientras espera)
 * \param data mdulo principal
 */
void player_state_ready (Player * data)
{

	data->actual_dx = 0;

	if (animation_update (data->animation))
		animation_reset (data->animation, READY);

	if (player_control_query (data->control, LEFT))
		player_do_run_left (data);

	if (player_control_query (data->control, RIGHT))
		player_do_run_right (data);

	if (player_control_query (data->control, UP))
		player_do_jump (data);
}


/*!
 * \brief Funcin que gestiona el estado SELECT (en la seleccin de personajes)
 * \param data mdulo principal
 */
void player_state_select (Player * data)
{
	animation_update (data->animation);
}


/*!
 * \brief Funcin que gestiona el estado RUN LEFT (caminado hacia la izquierda)
 * \param data mdulo principal
 */
void player_state_run_left (Player * data)
{
	if (animation_update (data->animation))
		animation_reset (data->animation, RUN_LEFT);
	
	player_move (data, - 1);

	if (! player_control_query (data->control, LEFT))
		player_do_ready (data);

	if (player_control_query (data->control, UP))
		player_do_jump (data);
}


/*!
 * \brief Funcin que gestiona el estado RUN RIGHT (caminado hacia la derecha)
 * \param data mdulo principal
 */
void player_state_run_right (Player * data)
{
	if (animation_update (data->animation))
		animation_reset (data->animation, RUN_RIGHT);

	player_move (data, + 1);

	if (! player_control_query (data->control, RIGHT))
		player_do_ready (data);

	if (player_control_query (data->control, UP))
		player_do_jump (data);
}


/*!
 * \brief Funcin que gestiona el estado JUMP (mientras salta)
 * \param data mdulo principal
 */
void player_state_jump (Player * data)
{
	data->y += data->initial_jump_speed >> 2;
	data->initial_jump_speed ++;

	if (data->y > FLOOR)
	{
		player_do_ready (data);
	}

	if (player_control_query (data->control, LEFT))
		player_move (data, - 1);

	if (player_control_query (data->control, RIGHT))
		player_move (data, + 1);

	animation_update (data->animation);
}


/*!
 * \brief Funcin que inicia el estado READY
 * \param data mdulo principal
 */
void player_init_ready (Player * data)
{
	data->actual_dx = 0;
	data->initial_jump_speed = 0;
	data->y = FLOOR;
	animation_reset (data->animation, READY);
}


/*!
 * \brief Funcin que inicia el estado SELECT
 * \param data mdulo principal
 */
void player_init_select (Player * data)
{
	animation_reset (data->animation, IN_SELECT);
}


/*!
 * \brief Funcin que inicia el estado RUN LEFT
 * \param data mdulo principal
 */
void player_init_run_left (Player * data)
{
	animation_reset (data->animation, RUN_LEFT);
}



/*!
 * \brief Funcin que inicia el estado RUN RIGHT
 * \param data mdulo principal
 */
void player_init_run_right (Player * data)
{
	animation_reset (data->animation, RUN_RIGHT);
}


/*!
 * \brief Funcin que inicia el estado JUMP
 * \param data mdulo principal
 */
void player_init_jump (Player * data)
{
	animation_reset (data->animation, JUMP);
	data->initial_jump_speed = - data->jump_power;
}


void player_move (Player * data, int direction)
{
	int dx = direction * data->speed;
	int ratio = (data->width + 1) / 2;
	int left = data->x - ratio;
	int right = data->x + ratio;

	if (direction < 0)
	{
		data->actual_dx = -1;

		if (left + dx < data->arena_boundary_left)
			data->x = data->arena_boundary_left + ratio;
		else
			data->x += dx;
	}
	else
	{
		data->actual_dx = +1;
		
		if (right + dx > data->arena_boundary_right)
			data->x = data->arena_boundary_right - ratio;
		else
			data->x += dx;
	}
}


/*!
 * \brief Especifica cuales son los lmites del escenario
 * \param data mdulo principal
 * \param left lmite izquierdo
 * \param right lmite derecho
 */
void player_set_arena_boudary (Player * data, int left, int right)
{
	data->arena_boundary_left = left;
	data->arena_boundary_right = right;
}


/*!
 * \brief Informa cual es el impulso que imprime a la pelota al colisionar
 * \param data mdulo principal
 * \param dx distancia relativa de la pelota y el personaje (x_ball - x_player)
 * \return poder a incrementar (puede ser negativo)
 */
float player_get_collision_power (Player * data, int dx)
{
	float suma = 1.1;
	float resta = 0.5;
	float normal = 0.8;
	float jumping = 0.5;

	if (data->initial_jump_speed == 0)
		jumping = 0;
	
	if (dx < 0)
	{
		if (data->actual_dx == 0)
			return normal + (jumping);
		else
			if (data->actual_dx > 0)
				return resta;
			else
				return + suma + jumping;
	}
	else
	{
		if (data->actual_dx == 0)
			return normal + jumping;
		else
			if (data->actual_dx > 0)
				return +suma + jumping;
			else
				return resta;
	}
	
	return 0;
}


/*!
 * \brief Informa el rea de colisin del personaje
 * \param data mdulo principal
 * \param x posicin horizontal
 * \param y posicin vertical
 * \param ratio radio de colisin
 */
void player_get_collision_area (Player * data, int * x, int * y, int * ratio)
{
	* y = data->y - data->hotspot_y + data->collision_y;
	* ratio = data->collision_ratio;
	
	if (data->flip)
		* x = data->x + data->hotspot_x - data->collision_x;
	else
		* x = data->x - data->hotspot_x + data->collision_x;
}

/*!
 * \brief Funcin llamada en caso de producirse una colisin
 * \param data mdulo principal
 * \param dx distancia relativa entre la pelota y el jugador
 */
void player_collision_with_ball (Player * data, int dx)
{
	int jumping = (data->initial_jump_speed != 0);
	int player = data->flip;
	enum animation_type type;

	/* define cual es la reaccin ante el toque de la pelota */
	
	if (dx < -25)
		type = TOUCH_BACK;
	else
	{
		if (dx > 25)
			type = TOUCH_FRONT;
		else
			type = TOUCH_UP;
	}

	/* invierte la direccin del golpe para el jugador 2 */
	if (player)
	{
		if (type == TOUCH_FRONT)
			type = TOUCH_BACK;
		else
		{
			if (type == TOUCH_BACK)
				type = TOUCH_FRONT;
		}
	}

	/* altera el tipo de golpe en caso de estar saltando */
	if (jumping)
	{
		if (type == TOUCH_UP)
			type = TOUCH_UP_JUMPING;
		else
		{
			if (type == TOUCH_BACK)
				type = TOUCH_BACK_JUMPING;
			else
				type = TOUCH_FRONT_JUMPING;
		}
	}
	
	animation_reset (data->animation, type);
}


/*!
 * \brief Cambia al estado RUN_LEFT
 * \param mdulo principal
 */
void player_do_run_left (Player * data)
{
	player_change_state (data, player_init_run_left, player_state_run_left);
}

/*!
 * \brief Cambia al estado RUN_RIGHT
 * \param data mdulo principal
 */
void player_do_run_right (Player * data)
{
	player_change_state (data, player_init_run_right, 
			player_state_run_right);
}

/*!
 * \brief Cambia al estado JUMP
 * \param data mdulo principal
 */
void player_do_jump (Player * data)
{
	player_change_state (data, player_init_jump, player_state_jump);
}

/*!
 * \brief Cambia al estado READY
 * \param data mdulo principal
 */
void player_do_ready (Player * data)
{
	player_change_state (data, player_init_ready, player_state_ready);
}
