/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.losersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "player_ai.h"
#include "ball.h"


/* 
 * The behavior of every player are driven by a state functions like 
 * `player_state_jump` or `player_state_run_left`.
 *
 * In this file you find a IA functions, that drive a player when some
 * scene call the function `player_set_ai_control`
 */



/*!
 * \brief Inicia el estado START
 * \param data mdulo principal
 */
void player_ai_init_start (Player * data)
{
	animation_reset (data->animation, READY);
}

/*!
 * \brief Gestiona el estado START
 * \param mdulo principal
 */
void player_ai_state_start (Player * data)
{
	if (ball_are_ready (data->ball))
	{
		if (player_ai_ball_are_in_my_side (data))
			player_ai_do_run_to_best_place_to_start (data);
		else
			player_ai_do_waiting_other_player (data);
	}
}


/*!
 * \brief Inicia el estado RUN_TO_BEST_PLACE_TO_START
 * \param mdulo principal
 */
void player_ai_init_run_to_best_place_to_start (Player * data)
{
}


/*!
 * \brief Gestiona el estado RUN_TO_BEST_PLACE_TO_START
 * \param mdulo principal
 */
void player_ai_state_run_to_best_place_to_start (Player * data)
{
	int done_run = player_ai_move_to_best_place_to_start (data);

	if (done_run)
		player_ai_do_start_game_jumping (data);
}




/*!
 * \brief Inicia el estado JUMP
 * \param data mdulo principal
 */
void player_ai_init_jump (Player * data)
{
	animation_reset (data->animation, JUMP);
	data->initial_jump_speed = - data->jump_power;
}


/*!
 * \brief Gestiona el estado JUMP
 * \param data mdulo principal
 */
void player_ai_state_jump (Player * data)
{
	data->y += data->initial_jump_speed >> 2;
	data->initial_jump_speed ++;
	
	animation_update_or_reset (data->animation, JUMP);

	/* add more power to punch */
	data->actual_dx = - player_ai_get_side (data);

	/* if done jump */
	if (data->y > FLOOR)
	{
		data->y = FLOOR;
		animation_reset (data->animation, READY);

		if (player_ai_ball_come_to_this_side (data))
			player_ai_do_waiting_receive (data);
		else
			player_ai_do_waiting_other_player (data);
	}
}

/*!
 * \brief Inicia el estado WAITING_OTHER_PLAYER
 * \param data mdulo principal
 */
void player_ai_init_waiting_other_player (Player * data)
{
}

/*!
 * \brief Gestiona el estado WAITING_OTHER_PLAYER
 * \param data mdulo principal
 */
void player_ai_state_waiting_other_player (Player * data)
{
	/* actualiza su posicin y animacin */
	if (player_ai_move_to_middle (data))
		animation_update_or_reset (data->animation, READY);
	
	if (player_ai_ball_are_in_my_side (data))
		player_ai_do_waiting_receive (data);
	else
	{
		if (ball_are_starting (data->ball))
			player_ai_do_start (data);
	}
}


/*!
 * \brief Inicia el estado WAITING_RECEIVE
 * \param data mdulo principal
 */
void player_ai_init_waiting_receive (Player * data)
{
}


/*!
 * \brief Gestiona el estado WAITING_RECEIVE
 * \param data mdulo principal
 */
void player_ai_state_waiting_receive (Player * data)
{
	/* si no pudo alcanzar la pelota y perdi la partida */
	if (ball_are_starting (data->ball))
	{
		player_ai_do_start (data);
		return;
	}

	/* si envi la pelota al otro lado del campo */
	if (! player_ai_ball_come_to_this_side (data))
	{
		player_ai_do_waiting_other_player (data);
		return;
	}
	
	/* Si puede golpear la pelota */
	if (player_ai_move_to_best_place_to_punch (data))
	{
		int ball_power = data->ball->position.power;
		int y_distance = player_ai_get_y_distance_to_ball (data);
		int x_distance = player_ai_get_x_distance_to_ball (data);

		if (ball_power < 5.0 && y_distance < 100 && x_distance < 30)
		{
			player_ai_do_jump (data);
			return;
		}

		animation_update_or_reset (data->animation, READY);
	}
}


/*!
 * \brief Realiza un cambio de estado rpido
 * \param data mdulo principal
 */
void player_ai_do_start (Player * data)
{
	void (* init) = player_ai_init_start;
	void (* update) = player_ai_state_start;
	
	debug_trace (data, "start");
	player_change_state (data, init, update);
}


/*!
 * \brief Realiza un cambio de estado rpido
 * \param data mdulo principal
 */
void player_ai_do_run_to_best_place_to_start (Player * data)
{
	void (* init) = player_ai_init_run_to_best_place_to_start;
	void (* update) = player_ai_state_run_to_best_place_to_start;

	debug_trace (data, "best_place_to_start");
	player_change_state (data, init, update);
}


/*!
 * \brief Realiza un cambio de estado rpido
 * \param data mdulo principal
 */
void player_ai_do_start_game_jumping (Player * data)
{
	void (* init) = player_ai_init_jump;
	void (* update) = player_ai_state_jump;

	debug_trace (data, "start_game_jumping");
	player_change_state (data, init, update);
}


/*!
 * \brief Realiza un cambio de estado rpido
 * \param data mdulo principal
 */
void player_ai_do_waiting_receive (Player * data)
{
	void (* init) = player_ai_init_waiting_receive;
	void (* update) = player_ai_state_waiting_receive;
	
	debug_trace (data, "waiting_receive");
	player_change_state (data, init, update);
}


/*!
 * \brief Realiza un cambio de estado rpido
 * \param data mdulo principal
 */
void player_ai_do_waiting_other_player (Player * data)
{
	void (* init) = player_ai_init_waiting_other_player;
	void (* update) = player_ai_state_waiting_other_player;

	debug_trace (data, "waiting_other_player");
	player_change_state (data, init, update);
}


/*!
 * \brief Realiza un cambio de estado rpido
 * \param data mdulo principal
 */
void player_ai_do_jump (Player * data)
{
	void (* init) = player_ai_init_jump;
	void (* update) = player_ai_state_jump;

	debug_trace (data, "jump");
	player_change_state (data, init, update);
}

/*!
 * \brief Verifica si la pelota ya se encuentra en su campo de juego
 * \param data mdulo principal
 * \return TRUE si est en su campo, FALSE si est del otro lado
 */
int player_ai_ball_are_in_my_side (Player * data)
{
	int left = data->arena_boundary_left;
	int right = data->arena_boundary_right;
	
	return util_include_value (data->ball->position.x, left, right);
}


/*!
 * \brief Obtiene la coordenada horizontal del cento de su campo de juego
 * \param mdulo principal
 * \return coordenada x del centro de su campo
 */
int player_ai_get_middle_of_side (Player * data)
{
	return (data->arena_boundary_right + data->arena_boundary_left) / 2;
}


/*!
 * \brief Verifica si la pelota est por llegar a este campo
 * \param data mdulo principal
 * \return TRUE si la pelota llegar, FALSE si an contina en campo adversario
 */
int player_ai_ball_come_to_this_side (Player * data)
{
	int last_x_position = ball_get_last_x_position_in_path (data->ball);
	int left = data->arena_boundary_left;
	int right = data->arena_boundary_right;

	if (ball_are_starting (data->ball))
		return FALSE;

	if (ball_no_path_available (data->ball))
		return FALSE;
	
	return util_include_value (last_x_position, left, right);
}


/*!
 * \brief Indica en que lado del campo se encuentra el jugador
 * \return -1 si se encuentra en el campo izquiedo y 1 en caso contrario
 */
int player_ai_get_side (Player * data)
{
	if (data->x > 320)
		return 1;
	else
		return -1;
}


/*!
 * \brief Indica si la pelota se encuentra en el otro campo
 * \param data mdulo principal
 * \return TRUE si est en el otro campo
 */
int player_ai_is_ball_in_the_other_side (Player * data)
{
	int left = data->arena_boundary_left;
	int right = data->arena_boundary_right;
	int ball_x = data->ball->position.x;
	
	return (ball_x < left || ball_x > right);
}


/*!
 * \brief Posiciona al jugador en algn punto de su campo
 * \param data mdulo principal
 * \param x posicin buscada
 * \param delta precisin para llegar a ese punto
 * \return TRUE si lleg al punto pedido, FALSE si contina caminando
 */
int player_ai_move_to (Player * data, int x, int delta)
{
	if (util_include_value (data->x, x - delta, x + delta))
		return TRUE;
	else
	{
		if (data->x < x)
		{
			animation_update_or_reset (data->animation, RUN_RIGHT);
			player_move (data, +1);
		}
		else
		{
			animation_update_or_reset (data->animation, RUN_LEFT);
			player_move (data, -1);
		}

		return FALSE;
	}
}

/*!
 * \brief Coloca al jugador en una posicin que le permita golpear la pelota
 * \return TRUE si encontr el lugar, FALSE si continua corriendo
 */
int player_ai_move_to_best_place_to_punch (Player * data)
{
	BallPosition ball_position;
	int x, y, r;
	int offset_to_punch;
	
	player_get_collision_area (data, & x, & y, & r);

	y -= r;

	ball_position = ball_get_punchable_position (data->ball, y);

	if (player_ai_get_side (data) == 1)
	{
		if (ball_position.x > 320 + 160)
			offset_to_punch = AI_DISTANCE_TO_PUNCH_FAR;
		else
			offset_to_punch = AI_DISTANCE_TO_PUNCH_NOT_FAR;

		data->actual_dx = -1;
	}
	else
	{
		if (ball_position.x < 160)
			offset_to_punch = -AI_DISTANCE_TO_PUNCH_FAR;
		else
			offset_to_punch = -AI_DISTANCE_TO_PUNCH_NOT_FAR;
		
		data->actual_dx = +1;
	}

	return player_ai_move_to (data, ball_position.x + offset_to_punch, 
			AI_MIN);
}


/*!
 * \brief Coloca al jugador en una posicin que le permita iniciar el juego
 * \return TRUE si encontr el lugar, FALSE si continua corriendo
 */
int player_ai_move_to_best_place_to_start (Player * data)
{
	int ball_x = player_ai_get_middle_of_side (data);
	int offset = player_ai_get_side (data) * util_rand (20, 25);
	int delta = PLAYER_AI_WANT_MOVE_DELTA;

	return player_ai_move_to (data, ball_x + offset, delta);
	
	/*
	if (util_include_value (data->x, ball_x - delta, ball_x + delta))
	{
		return TRUE;
	}
	else
	{
		if (data->x < ball_x)
		{
			animation_update_or_reset (data->animation, RUN_RIGHT);
			player_move (data, +1);
		}
		else
		{
			animation_update_or_reset (data->animation, RUN_LEFT);
			player_move (data, -1);
		}

		return FALSE;
	}
	*/
}



/*!
 * \brief Posiciona al jugador en el centro de su campo
 * \param data mdulo principal
 * \return TRUE si alcanz esa posicin, FALSE en caso contrario
 */
int player_ai_move_to_middle (Player * data)
{
	int middle = player_ai_get_middle_of_side (data);
	
	return player_ai_move_to (data, middle, PLAYER_AI_WANT_MOVE_DELTA);
}




int player_ai_get_y_distance_to_ball (Player * data)
{
	int x;
	int y;
	int ratio;

	player_get_collision_area (data, & x, & y, &ratio);
	return abs (y - data->ball->position.y);
}

int player_ai_get_x_distance_to_ball (Player * data)
{
	int x;
	int y;
	int ratio;

	player_get_collision_area (data, & x, & y, &ratio);
	return abs (x - data->ball->position.x);
}
