/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "select.h"
#include "display.h"
#include "main_menu.h"
#include "player.h"

/*!
 * \brief Genera el mdulo de escena `Select`
 * \param world mdulo principal del programa
 * \return el mdulo creado, o aborta el programa si ocurre un error
 */
Select * select_init (World * world)
{
	Select * data;
	int max_players;

	data = (Select *) malloc (sizeof (Select));

	if (! data)
	{
		util_print_fail_module ("Select");
		exit (1);
	}

	data->vs_mode = FALSE;
	data->world = world;
	select_draw_background (data);
	
	max_players = characters_count_players (world->characters);

	data->cursor_1 = cursor_init (world, 0, max_players);
	
	return data;
}

/*!
 * \brief Define el fondo de pantalla
 * \param data mdulo principal
 */
void select_draw_background (Select * data)
{
	SDL_Surface * background;

	background = util_load_image (DATADIR "/select.png", FALSE);

	if (background)
	{
		display_set_background (data->world->display, background);
		SDL_FreeSurface (background);
	}

}


/*!
 * \brief Imprime la grilla de personajes
 * \param data mdulo principal
 */
void select_draw_characters (Select * data)
{
	Player * player;
	int i = 0;

	node_iterator_next (data->world->characters->list);

	player = (Player *) node_iterator_next (NULL);

	while (player)
	{
		int x = 160 - 32 + (i % COLUMNS) * 64;
		int y = 200 - 32 + (i / COLUMNS) * 64;
		
		player_draw_preview (player, x, y, data->world->display);
		
		i ++;
		player = (Player *) node_iterator_next (NULL);
	}
}


/*!
 * \brief Actualizacin lgica de la escena
 * \param data mdulo principal
 */
void select_update (Select * data)
{
	cursor_update (data->cursor_1);

	if (cursor_selected (data->cursor_1))
		select_init_game (data);

	if (control_query (data->world->control, QUIT))
		select_return_to_main (data, 0);
}

/*!
 * \brief Actualizacin grafica
 * \param data mdulo principal
 */
void select_draw (Select * data)
{
	select_draw_characters (data);
	cursor_draw (data->cursor_1, data->world->display);
	font_printf (data->world->font, data->world->display, 10, 10, FALSE, 
			_("Player Select"));
}

/*!
 * \brief Elimina el mdulo `Select`
 * \param data mdulo principal
 */
void select_free (Select * data)
{
	cursor_free (data->cursor_1);
	free (data);
}

/*!
 * \brief termina la escena Select para iniciar el juego
 * \param data mdulo principal
 */
void select_init_game (Select * data)
{
	Game * game;
	Player * p1 = cursor_get_player (data->cursor_1);
	Player * p2 = NULL;
	
	do
	{
		p2 = characters_get_random_player (data->world->characters);
	} 
	while (p2 == p1);

	player_set_control (p1, data->world->player_control_1);
	player_set_ai_control (p2);

	game = game_init (data->world, p1, p2);

	world_change_scene (data->world, game, game_update, game_draw, 
			game_free);
}


void select_return_to_main (Select * data, int active_entry)
{
	Main_menu * menu = main_menu_init (data->world, active_entry);
	
	world_change_scene (data->world, menu, main_menu_update, 
			main_menu_draw, main_menu_free);
}
