/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "util.h"
#include <time.h>

/*!
 * \brief imprime en pantalla el mensaje `fallo la creacin del mdulo X`
 * \param module_name identificador del mdulo
 */
void util_print_fail_module (char * module_name)
{
	printf (_("Cant' create a %s module\n"), module_name);
}

/*!
 * \brief Carga y optimiza una imgen
 * \param file ruta completa hacia el archivo
 * \param with_alpha TRUE si la imagen tiene incorporado el canal de transparencia, como ocurre generalmente con las imgenes .PNG. Indique FALSE si la imagen debe utilizar el color (255, 0, 255) como transparente.
 */
SDL_Surface * util_load_image (const char * file, int with_alpha)
{
	SDL_Surface * tmp;
	SDL_Surface * surface;

	tmp = IMG_Load (file);

	if (! tmp)
	{
		printf (_("Can't load '%s' file: %s\n"), file, SDL_GetError ());
		return NULL;
	}

	if (with_alpha)
	{
		surface = SDL_DisplayFormatAlpha (tmp);

		if (surface)
		{
			SDL_FreeSurface (tmp);
			return surface;
		}
		else
			return tmp;
	}
	else
	{
		Uint32 color = SDL_MapRGB (tmp->format, 255, 0, 255);
		
		SDL_SetColorKey (tmp, SDL_SRCCOLORKEY, color);

		surface = SDL_DisplayFormat (tmp);

		if (surface)
		{
			SDL_FreeSurface (tmp);
			return surface;
		}
		else
			return tmp;
	}
}


/*!
 * \brief imprime los crditos del programa y los trminos de licencia
 */
void util_print_welcome (void)
{
	printf (_("Starting: %s (%s)\n"), "Pocket Volleyball", PACKAGE_STRING);
	printf (_("Copyright (C) %s"), "2005-2006 Hugo Ruscitti\n");
	printf (_("Web site: %s\n"), "http://www.losersjuegos.com.ar");
	printf ("\n");
	printf (_("This is free software; see the source for copying\n"
		  "conditions. There is NO warranty; not even for \n"
		  "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"));
	
	printf ("\n");
	printf (_("Remember run this program with --help for more options\n"));
}


/*!
 * \brief imprime una pequea ayuda
 * \param programa nombre del fichero ejecutable
 */
void util_print_help (const char * program)
{
	printf (_("This is %s. Usage:\n"), "Pocket Volleyball");
	printf ("\n");
	printf ("  %s [OPTIONS]\n", program);
	printf ("\n");
	printf ("Options:\n");

	printf ("  -h, --help\t\t\t%s", _("Display this help message\n"));
	printf ("  -f, --fullscreen\t\t%s", _("Enable Fullscreen mode\n"));
	printf ("  -d, --debug\t\t\t%s", _("Enable Debug mode\n"));
	printf ("  -da, --disable-audio\t\t\t%s", _("Disable Audio system\n"));
	printf ("  -s #      \t\t\t%s", _("Set speed game: # frames per second\n"));
	printf ("\n");
}


/*!
 * \brief informa si un archivo existe y es posible leerlo
 * \param ruta direccin completa hacia el archivo
 * \return 1 si existe, 0 en otro caso
 */
int util_file_exist (char * base_dir, char * name)
{
	char path [1024];
	FILE * tmp;

	sprintf (path, "%s/%s", base_dir, name);

	tmp = fopen (path, "r");

	if (tmp)
	{
		fclose (tmp);
		return 1;
	}
	else
		return 0;
}


/*!
 * \brief informa si existe el directorio apuntado por ruta
 * \param ruta direccin al directorio
 * \return 1 si existe, 0 en otro caso
 */
int util_directory_exist (char * path)
{
	DIR * tmp;

	tmp = opendir (path);

	if (tmp)
	{
		closedir (tmp);
		return 1;
	}

	return 0;
}


void util_remove_breakline (char * buffer)
{
	if (buffer [strlen (buffer) - 1] == '\n')
		buffer [strlen (buffer) - 1] = '\0';
}


/*!
 * \brief Elimina los espacios al principio y al final de una cadena
 * \param buffer cadena a modificar
 */
void util_remove_spaces (char * buffer)
{
	int r;
	int w = 0;
	int start = 1;
	int i;

	/* elimina los primeros espacios */
	for (r = 0; buffer [r]; r ++)
	{
		buffer [w] = buffer [r];
		
		if (start)
		{
			if (buffer [r] != ' ')
			{
				w ++;
				start = 0;
			}
		}
		else
			w ++;
	}

	buffer [w] = buffer [r];

	/* elimina los espacios al final de la palabra */
	for (i = strlen (buffer) - 1; i > 0; i --)
	{
		if (buffer [i] == ' ')
			buffer [i] = '\0';
		else
			return;
	}
}


/*!
 * \brief Obtiene un pixel de una superficie
 * \param ima superficie para analizar
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \return pixel seleccionado
 */
Uint32 util_get_pixel (SDL_Surface * ima, int x, int y)
{
	int bpp = ima->format->BytesPerPixel;
	Uint8 *p = (Uint8 *) ima->pixels + y * ima->pitch + x * bpp;

	switch (bpp)
	{
		case 1:
			return *p;
		
		case 2:
			return *(Uint16 *)p;

		case 3:
			if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;

		case 4:
			return *(Uint32 *)p;

		default:
			return 0;
	}
}



/*!
 * \brief imprime un punto de color sobre la superficie del primer parmetro
 * \param ima superficie destino
 * \param x coordenada horizontal
 * \param y coordenada vertical
 * \param color color del punto
 */
void util_put_pixel (SDL_Surface * ima, int x, int y, Uint32 color)
{
	int bpp = ima->format->BytesPerPixel;
	Uint8 *p = (Uint8 *) ima->pixels + y * ima->pitch + x * bpp;

	if (x >= ima->w || y >= ima->h || y < 0 || x < 0)
		return;

	switch (bpp)
	{
		case 1:
			*p = color;
		break;
		
		case 2:
			*(Uint16 *)p = color;
		break;

		case 3:
			if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
			{
				p[0] = (color >> 16) & 0xff;
				p[1] = (color >> 8) & 0xff;
				p[2] = color & 0xff;
			}
			else
			{
				p[0] = color & 0xff;
				p[1] = (color >> 8) & 0xff;
				p[2] = (color >> 16) & 0xff;
			}
		break;

		case 4:
			* (Uint32 *)p  = color;
		break;
	}
}


/*!
 * \brief Imprime una circunfencia sobre `ima`
 * \param ima superficie a modificar
 * \param color color de trazo
 * \param cx coordenada horizontal del centro de circunferencia
 * \param cy coordenada vertical del centro de circunferencia
 * \param ratio radio de la circunferencia
 */
void util_draw_circle (SDL_Surface * ima, Uint32 color, int cx, int cy, int r)
{
	int x;
        int y = r;

        if (SDL_MUSTLOCK (ima))
                SDL_LockSurface (ima);

        for (x = 0; x < y; x ++)
        {
		y = (int) sqrt (r * r - x * x);

		/* arcos de cuadrantes, del primero al cuarto (en ese orden) */
		util_put_pixel (ima, cx + x, cy - y, color);
		util_put_pixel (ima, cx + y, cy - x, color);

		util_put_pixel (ima, cx + x, cy + y, color);
		util_put_pixel (ima, cx + y, cy + x, color);

		util_put_pixel (ima, cx - x, cy + y, color);
		util_put_pixel (ima, cx - y, cy + x, color);

		util_put_pixel (ima, cx - x, cy - y, color);
		util_put_pixel (ima, cx - y, cy - x, color);
	}
	        
	if (SDL_MUSTLOCK (ima))
		SDL_UnlockSurface (ima);
}


/*!
 * \brief Determina si existe una colisin entre dos circunferencias
 * \param x0 coordenada horizontal del primer centro de colisin
 * \param y0 coordenada vertical del primer centro de colisin
 * \param x0 coordenada horizontal del segundo centro de colisin
 * \param y0 coordenada vertical del segundo centro de colisin
 * \param ratio_sum suma de ambos radios de circunferencia
 */
int util_is_collision (int x0, int y0, int x1, int y1, int ratio_sum)
{
	int x = abs (x0 - x1);
	int y = abs (y0 - y1);

	return (x * x + y * y < ratio_sum * ratio_sum);
}

/*!
 * \brief calcula el angulo que existe entre 2 puntos
 * \param x0 coordenada horizontal del primer punto
 * \param y0 coordenada vertical del primer punto
 * \param x1 coordenada horizontal del segundo punto
 * \param y1 coordenada vertical del segundo punto
 * \return el ngulo en radianes 
 */
float util_get_angle (int x0, int y0, int x1, int y1)
{
	int dx = x0 - x1;
	int dy = y0 - y1;

	return (atan2 (- dy, dx));
}

/*!
 * \brief Informa la distancia entre dos puntos de la pantalla
 * \param x0 coordenada horizontal del primer punto
 * \param y0 coordenada vertical del primer punto
 * \param x1 coordenada horizontal del segundo punto
 * \param y1 coordenada vertical del segundo punto
 */
int util_get_distance (int x0, int y0, int x1, int y1)
{
	float dx = x1 - x0;
	float dy = y1 - y0;

        return (sqrt (dy * dy + dx * dx));
}

/*!
 * \brief informa si un valor se encuentra comprendido entre otros 2
 * \param value punto a evaluar
 * \param left_value limite de menor valor
 * \param right_value limite de mayor valor
 * \return TRUE si x est en el rango [inferior, superior], FALSE en otro caso
 */
int util_include_value (int value, int left_value, int right_value)
{
	return (value >= left_value && value <= right_value);
}


/*!
 * \brief Inicia el generador de nmeros aleatorios
 */
void util_start_rand (void)
{
	srand (time (NULL));
}


/*!
 * \brief selecciona un nmero entre a y b al azar
 * \param a cota inferior
 * \param b cota superior
 * \return nmero aleatorio perteneciente a [a, b]
 */
int util_rand (int a, int b)
{
	return rand () % (b -a + 1) +  a;
}


/*!
 * \brief Genera una superficie simulando una proyeccin de sombra
 * \param src imagen para proyectar
 */
SDL_Surface * util_create_shadow (SDL_Surface * src, int cols)
{
	SDL_Surface * dst;
	Uint32 color;
	int i;
	int tile_width = src->w / cols;
	int new_tile_width = tile_width + 50;

	
	dst = SDL_CreateRGBSurface (src->flags, new_tile_width * cols, 
			src->h,
			src->format->BitsPerPixel, src->format->Rmask, 
			src->format->Gmask, src->format->Bmask, 
			src->format->Amask);

	if (! dst)
	{
		printf ("Error: %s\n", SDL_GetError ());
		return NULL;
	}


	/* limpia la superficie por completo */
	color = SDL_MapRGB (dst->format, 255, 30, 255);
	SDL_FillRect (dst, NULL, color);
	SDL_SetColorKey (dst, SDL_SRCCOLORKEY | SDL_RLEACCEL, color);

	/*
	for (i = 0; i < cols; i ++)
		util_draw_shadow (src, i, cols, dst, 50 + i * new_tile_width,0);
	*/

	SDL_SetAlpha (dst, SDL_SRCALPHA, 128);
	return dst;
}


void util_draw_shadow (SDL_Surface * src, int frame, int columns, 
		SDL_Surface * dst, int x, int y, int flip)
{
	int tile_width = src->w / columns;
	int tile_height = src->h;
	int frame_x = tile_width * frame;
	Uint32 shadow_color = SDL_MapRGB (dst->format, 0, 0, 0);
	int dx;
	int dy;

	/* genera la transformacin copiando puntos de la imgen original */
	for (dx = 0; dx < tile_width; dx ++)
	{
		for (dy = 0; dy < tile_height; dy += 2)
		{
			Uint32 pixel;
			SDL_Color color;
			int actual_dx = (dy - src->h);

			pixel = util_get_pixel (src, frame_x + dx, dy);

			SDL_GetRGB (pixel, src->format, & color.r, 
					& color.g, & color.b);

			if (pixel != src->format->colorkey)
			{
				util_put_pixel (dst, x + dx + actual_dx, 
						y + dy, shadow_color);
			}
		}
	}
}
