/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "world.h"
#include "common.h"

/*!
 * \brief Inicia el mdulo
 * \param argc argumentos de programa
 * \param argv argumentos de programa
 * \return el mdulo creado o NULL en caso de error
 */
World * world_init (int argc, char * argv [])
{
	World * data;

	data = (World *) malloc (sizeof (World));

	if (! data)
	{
		util_print_fail_module ("World");
		return NULL;
	}

	if (SDL_Init (SDL_INIT_VIDEO))
	{
		printf (_("Can't init SDL: %s\n"), SDL_GetError ());
		return NULL;
	}

	data->options = options_init ();

	if (world_parse_arguments (data, argc, argv))
		return NULL;

	data->display = display_init (data->options->fullscreen);
	data->fps = fps_init (data->options->fps, data->options->debug);
	data->characters = characters_init (data->options);
	
	data->font = font_init (DATADIR "/font.png", 
			DATADIR "/font_selected.png", 
			DATADIR "/font_disabled.png");
	
	data->control = control_init ();
	data->player_control_1 = player_control_init (data->control, 0);
	data->player_control_2 = player_control_init (data->control, 1);
	
	data->audio = audio_init (data->options->audio_enabled);
	
	data->quit = FALSE;
	data->scene = NULL;
	data->draw_scene = NULL;
	data->update_scene = NULL;
	data->free_scene = NULL;

	return data;
}


/*!
 * \brief Analiza los argumentos de programa
 * \param data mdulo principal
 * \param argc argumentos de programa
 * \param argv argumentos de programa
 * \param fs si define modo pantalla completa o no
 * \return 1 si el programa debe terminar (error o --help), 0 en otro caso
 */
int world_parse_arguments (World * data, int argc, char * argv [])
{
	int i;

	for (i = 0; i < argc; i ++)
	{
		if (strcmp (argv [i], "--help") == 0 
				|| strcmp (argv [i], "-h") == 0)
		{
			util_print_help (argv [0]);
			return 1;
		}

		if (strcmp (argv [i], "--fullscreen") == 0
				|| strcmp (argv [i], "-f") == 0)
		{
			data->options->fullscreen = TRUE;
		}

		if (strcmp (argv [i], "--debug") == 0 
				|| strcmp (argv [i], "-d") == 0)
		{
			data->options->debug = TRUE;
		}

		if (strcmp (argv [i], "--disable-audio") == 0 
				|| strcmp (argv [i], "-da") == 0)
		{
			data->options->audio_enabled = FALSE;
		}

		
		if (strcmp (argv [i], "-s") == 0)
		{
			data->options->fps = atoi (argv [i + 1]);
		}
	}

	return 0;
}


/*!
 * \brief Bucle principal, mantiene el programa en constante actualizacin
 * \param data mdulo principal
 */
void world_main_loop (World * data)
{
	int i;
	int logic_updates;

	while (! data->quit)
	{
		world_poll_events (data);
		
		logic_updates = fps_synchronize (data->fps);

		if (logic_updates)
		{
			for (i = 0; i < logic_updates; i ++)
			{
				control_update (data->control);
				data->update_scene (data->scene);
			}

			data->draw_scene (data->scene);
			display_update (data->display);
		}
	}
}


/*!
 * \brief Termina la ejecucin del mdulo
 * \param data mdulo principal
 */
void world_free (World * data)
{
	if (data->scene)
		data->free_scene (data->scene);

	display_free (data->display);
	fps_free (data->fps);
	characters_free (data->characters);
	font_free (data->font);
	control_free (data->control);
	player_control_free (data->player_control_1);
	player_control_free (data->player_control_2);
	audio_free (data->audio);
	
	SDL_Quit ();
}


/*!
 * \brief Actualiza la cola de eventos capturando a los mas importantes
 * \param data mdulo principal
 */
void world_poll_events (World * data)
{
	while (SDL_PollEvent (& data->event))
	{
		if (data->event.type == SDL_QUIT)
			data->quit = 1;

		if (data->event.type == SDL_KEYDOWN)
		{
			switch (data->event.key.keysym.sym)
			{
				case SDLK_f:
					display_enable_fullscreen 
						(data->display);
					break;

				case SDLK_n:
					display_disable_fullscreen 
						(data->display);
					break;
					
				default:
					break;
			}
		}
	}
}


/*!
 * \brief Altera la escena que gestiona `World`
 * \param data mdulo principal
 * \param scene mdulo que representa al escena
 * \param update_scene funcin que actualizar la escena
 * \param draw_scene funcin que realiza la actualizacin grfica de la escena
 * \param free_scene funcion que elimina la escena de memoria
 */
void world_change_scene (World * data, void * scene, void (* update_scene), 
		void (* draw_scene), void (* free_scene))
{
	if (data->scene)
	{
		data->free_scene (data->scene);
	}

	data->scene = scene;
	data->update_scene = update_scene;
	data->draw_scene = draw_scene;
	data->free_scene = free_scene;

	fps_reset (data->fps);
}

/*!
 * \brief termina la ejecucin del programa
 * \param data mdulo principal
 */
void world_quit (World * data)
{
	data->quit = TRUE;
}
