/*
 * Volleyball - video game similary to GNU Arcade Volleyball
 * Copyright (C) 2005, 2006 Hugo Ruscitti : hugoruscitti@gmail.com
 * web site: http://www.loosersjuegos.com.ar
 * 
 * This file is part of Volleyball.
 *
 * Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef _WORLD_H
#define _WORLD_H

/*!
 * \file
 */

#include "common.h"
#include "fps.h"
#include "display.h"
#include "characters.h"
#include "options.h"
#include "font.h"
#include "control.h"
#include "player_control.h"
#include "audio.h"

/*!
 *
 * \brief Mdulo pricipal del programa
 *
 * Administra las Escenas del juego, inicializa la biblioteca y se 
 * encarga de controlar la velocidad constante del programa en 
 * cualquier tipo de equipos.
 *
 * Para administrar las escenas del juego, Mundo debe conocer a la 
 * escena actual. A medida que el programa se ejecuta, World llama a 
 * los punteros de funcin 'update' y 'display'. Estos se encargan de 
 * operar con la escena actual.
 *
 * Es importante indicarle a World cual es la escena que debe ejecutar, 
 * indicando las funciones `update`, `display` y `quit` de la escena
 * actual llamando a la funcion `world_change_scene`. El primer llamado
 * a la funcin `world_change_scene` se realiza en la funcin `main`, 
 * pero luego, cada escena conocer el mdulo `world` y podr intercambiar
 * a otras escenas. Por ejemplo, desde el `Menu` principal se llama a la
 * funcin `world_change_scene` para ingresar al juego.
 * 
 * World delega al tarea de gestionar la pantalla al mdulo `Display`, por
 * lo tanto al iniciar el mdulo se le solicita a `Display` que inicie la 
 * ventana.
 */
typedef struct world_t
{
	Uint8 quit;		/*!< indica si el programa debe terminar */
	SDL_Event event;	/*!< eventos generales de la aplicacin */
	Display * display;		/*!< gestor de pantalla */
	Fps * fps;			/*!< gestor de velocidad constante */
	Characters * characters;	/*!< grupo de personajes disponibles */
	Options * options;			/*!< opciones del programa */
	Font * font;				/*!< mdulo de fuentes */
	Control * control;			/*!< controlador de teclado */
	Player_control * player_control_1; 	/*!< controla al jugador 1 */
	Player_control * player_control_2; 	/*!< controla al jugador 2 */
	Audio * audio;
	
	void * scene;				/*!< escena actual */
	void (* update_scene) (void * screne);	/*!< actualizacin lgica 
						  de la escena */
	void (* draw_scene) (void * screne);	/*!< actualizacin grfica de 
						  la escena */
	void (* free_scene) (void * screne);	/*!< libera la escena actual */
} World;


World * world_init (int argc, char * argv []);
int world_parse_arguments (World * data, int argc, char * argv []);
void world_main_loop (World * data);
void world_free (World * data);
void world_poll_events (World * data);
void world_change_scene (World * data, void * scene, void (* update_scene), 
		void (* draw_scene), void (* free_scene));
void world_quit (World * data);

#endif
